--camera by Michael Ebens 
camera = {}
camera._x = 0
camera._y = 0
camera.scaleX = 1
camera.scaleY = 1
camera.rotation = 0

function camera:set()
  love.graphics.push()
  love.graphics.rotate(-self.rotation)
  love.graphics.scale(1 / self.scaleX, 1 / self.scaleY)
  love.graphics.translate(-self._x, -self._y)
end

function camera:unset()
  love.graphics.pop()
end

function camera:move(dx, dy)
  self._x = self._x + (dx or 0)
  self._y = self._y + (dy or 0)
end

function camera:rotate(dr)
  self.rotation = self.rotation + dr
end

function camera:scale(sx, sy)
  sx = sx or 1
  self.scaleX = self.scaleX * sx
  self.scaleY = self.scaleY * (sy or sx)
end

function camera:setX(value)
  if self._bounds then
    self._x = math.clamp(value, self._bounds.x1, self._bounds.x2)
  else
    self._x = value
  end
end

function camera:setY(value)
  if self._bounds then
    self._y = math.clamp(value, self._bounds.y1, self._bounds.y2)
  else
    self._y = value
  end
end

function camera:setPosition(x, y)
  if x then self:setX(x) end
  if y then self:setY(y) end
end

function camera:setScale(sx, sy)
  self.scaleX = sx or self.scaleX
  self.scaleY = sy or self.scaleY
end

function camera:getBounds()
  return unpack(self._bounds)
end

function camera:setBounds(x1, y1, x2, y2)
  self._bounds = { x1 = x1, y1 = y1, x2 = x2, y2 = y2 }
end

function math.clamp(x, min, max)
  return x < min and min or (x > max and max or x)
end

--/camera

--tabletostring by lua-users wiki (http://lua-users.org/wiki/TableUtils)
function table.val_to_str ( v )
  if "string" == type( v ) then
    v = string.gsub( v, "\n", "\\n" )
    if string.match( string.gsub(v,"[^'\"]",""), '^"+$' ) then
      return "'" .. v .. "'"
    end
    return '"' .. string.gsub(v,'"', '\\"' ) .. '"'
  else
    return "table" == type( v ) and table.tostring( v ) or
      tostring( v )
  end
end

function table.key_to_str ( k )
  if "string" == type( k ) and string.match( k, "^[_%a][_%a%d]*$" ) then
    return k
  else
    return "[" .. table.val_to_str( k ) .. "]"
  end
end

function table.tostring( tbl )
  local result, done = {}, {}
  for k, v in ipairs( tbl ) do
    table.insert( result, table.val_to_str( v ) )
    done[ k ] = true
  end
  for k, v in pairs( tbl ) do
    if not done[ k ] then
      table.insert( result,
        table.key_to_str( k ) .. "=" .. table.val_to_str( v ) )
    end
  end
  return "{" .. table.concat( result, "," ) .. "}"
end
--/tabletostring

--main code by Wojak 
-- game states: -1 - .conf file restoring, 0 - .conf file processing, 1 - menu, 2 - game runing, 3 - pause, 4 - victory screen, 5 - fail screen
--		6 - display score, 7 - set score, 8 - map editor, 9 - maap save, 10 - map test, 11 - about text

-- this should be love.load:

love.graphics.setBackgroundColor( 0, 250, 0)
local dataframe = love.image.newImageData(32,32)
for i=0, 31 do  
	for j=0,31 do
		if i == 0 or i == 31 or j == 0 or j == 31 then
   			dataframe:setPixel(i, j, 0, 0, 0, 255)
		else
			dataframe:setPixel(i, j, 0, 0, 0, 0)
		end
	end
end
local imageData = love.image.newImageData( "-atlas.png" )
for i=0,159 do
	for j=0,159 do
		local r,g,b,a = imageData:getPixel(i,j)
		if r == 255 and g == 255 and b == 255 and a == 255 then
			imageData:setPixel(i,j,255,255,255,0)
		end
	end
end
local imageframe = love.graphics.newImage(dataframe)
local imagegrid = imageframe 

local imageatlas = love.graphics.newImage(imageData)
local imagebackground = imageatlas

local imageplayer = love.graphics.newQuad(0, 96, 32, 32, 160, 160)
local imagebot = love.graphics.newQuad(32, 96, 32, 32, 160, 160)
local imagewall = love.graphics.newQuad(96, 0, 32, 32, 160, 160)
local imagewat = love.graphics.newQuad(32, 32, 32, 32, 160, 160)
local imagebri1 = love.graphics.newQuad(0, 0, 32, 32, 160, 160)
local imagebri2 = love.graphics.newQuad(32, 0, 32, 32, 160, 160)
local imagebri3 = love.graphics.newQuad(64, 0, 32, 32, 160, 160)
local imageice = love.graphics.newQuad(128, 0, 32, 32, 160, 160)
local imagebus = love.graphics.newQuad(128, 32, 32, 32, 160, 160)
local imagemud = love.graphics.newQuad(64, 32, 32, 32, 160, 160)
local imagelaw = love.graphics.newQuad(0, 32, 32, 32, 160, 160)
local imagefir = love.graphics.newQuad(96, 32, 32, 32, 160, 160)
local imagegras = love.graphics.newQuad(0, 64, 32, 32, 160, 160)
local imagebri4 = love.graphics.newQuad(32, 64, 32, 32, 160, 160)
local iamgebasepla = love.graphics.newQuad(64, 96, 32, 32, 160, 160)
local imagebasebot = love.graphics.newQuad(96, 96, 32, 32, 160, 160)
love.filesystem.setIdentity( "tanksreloaded" )
local botsoushoot
local bulletsouhit
local soushoot
local souhit
local souboom 
local souride 
local scalex = 1
local scaley = 1
local scalexbus = 1
local scaleybus = 1
local isup = false
local isdown = false
local isleft = false
local isright = false
local interval = 0.1
local botinterval = 1
local udt = 0
local ddt = 0
local ldt = 0 
local rdt = 0
local botdt = 0
local screentimer = 5
local screendt = 0
local player
local map
local dynamicmap
local bottab = {}
local bullettab = {}
local basetab = {}
local smallmapsize = 400
local slallbotn = 10
local mediummapsize = 900
local mediumbotn = 15
local largemapsize = 2030
local largebotn = 30
local verylalgebotn = 50
local botn = 15
local botleft = 5
local playerbasehp = 5
local botbasehp = 5
local storebotleft = botleft
local botincreeseamount = 5
local botincreese = 0
local bulletandisplay = "ON"
local firedetectordisplay = "ON"
local width = love.graphics.getWidth()
local height = love.graphics.getHeight()
local gamestate = 0
local menustate = 2
local cursorstate =  height/2 -8 * 32
local shiftstate = false
local playsound = false
local playsounddisplay = "OFF"
local spawnmap = {}
local testtest = ""
local nmenu = false
local winstate = ""
local mapid = 1
local mapdraw = "1"
local directory = love.filesystem.getWorkingDirectory( )
local curmenu ={}
local allmenu ={}
local pausemenutab = {}
local movetab = {}
local editorcursor = {
	mapposx=width / 2,
	mapposy=height / 2,
	x=2,
	y=2,
	id = -1,
	blink = 1,
	blinkstate = false,
	plaspawn = {},
	botspawn = {},
	basespawn = {},
	name = "",
	errorname = ""
}

local navtab = {}
local scoretab = {}
local specialkeytab = {}
local fileexsist, ok, chunk, result
local confprocesstab = {}
local spottab = {}
local tempspottab = {}

local savedir = love.filesystem.getSaveDirectory( )
--table.insert(confprocesstab,#confprocesstab + 1,savedir.."tank.conf")
--table.insert(confprocesstab,#confprocesstab + 1,tostring(type(souride)))
local allmenu = {}

fileexsist = io.open(savedir.."/score.txt", "r")
if not fileexsist then
	local text = love.filesystem.read("scoreform.txt") 
	love.filesystem.write("score.txt",text)
end
fileexsist = io.open(savedir.."/maplist.bck", "r")
if not fileexsist then
	local text = love.filesystem.read("maplist.bck") 
	love.filesystem.write("maplist.bck",text)
end
local func = love.filesystem.load("score.txt")
scoretab = func()

local readkey = flase

local mp1 = {
up = "up",
down = "down",
left = "left",
right = "right",
shoot = "return"
}
local mm0 = {
up = "up",

down = "down",

left = "left",

right = "right",

action = "return",

pause = "escape"
}
local mp2 = {
up = "w",

down = "s",

left = "a",

right = "d",

shoot = "r"
}
local me0 = {
up = "up",

down = "down",

left = "left",

right = "right",

next = "pageup",
previous = "pagedown",
save_quit = "escape"
}
local deftablst = {[mm0]="menu_keys",[mp1]="player1_keys",[mp2]="player2_keys",[me0]="editor_keys"}
function getMaps()
	return {"1.map","2.map","3.map","4.map","5.map","6.map","7.map","8.map","9.map","10.map","11.map","12.map","13.map","14.map","15.map","16.map","17.map",}
end
local maps = getMaps()
local gamemodelst = {"Kill them all","Base defence","Assault","Assault and defance"}
local gamemodestate = 1
--map ids properties (edit-30-03-12)
local hasidcolision = {[-3]=true,[1]=true,[2] = true, [3]=true,[4]=true,[5]=true,[11]=true,[12]=true} 
local hasidcolisionforbullets = {[-3]=true,[2] = true, [3]=true,[4]=true,[5]=true,[11]=true,[12]=true} 
local hasidslowdowneffect = {[7]=1.2,[8]=1.2,[9]=1} 
local hasidsliperyeffect = {[6]=2,[8]=1,[9]=-1,[10]=-1} 
local hasiddamegeeffect = {[9]=2,[10]=1} 
local isdynamicidplayer = {[3] = true}
local isdynamicidbot = {[1] = true}
local isidbrick = {[3] = true,[4]=true,[5]=true}
local mapidtab = {[-3]=iamgebasepla,[-2]=imagebot,[-1]=imageplayer,[0]=imagegras,[1]=imagewat,[2]=imagewall,[3]=imagebri1,[4]=imagebri2,[5]=imagebri3,[6]=imageice,[7]=imagebus,[8]=imagemud,[9]=imagelaw,[10]=imagefir,[11]=imagebri4,[12]=imagebri4}
---

-- list of all (i hope) keys  "
local keytab = {"0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "a", "b", "c", "d", "e", "f", "g", "h", "i", "j", "k",
 "l", "m", "n", "o", "p", "q", "r", "s", "t", "u", "v", "w", "x", "y", "z","F1", "F2", "F3", "F4", "F5",
 "F6", "F7", "F8", "F9", "F10", "F11", "F12", "backspace", "tab", "lalt", "ralt", "return", " ", "pageup", "pagedown", "end", "home", "escape",
 "insert", "delete", "lshift", "rshift", "lctrl", "rctrl", "[", "]", "pause", "capslock", "scroll", ";", ",", "-", ".", "/",
 "#", "\\", "=","up", "down", "left", "right", "kp0","kp1","kp2","kp3","kp4","kp5","kp6","kp7","kp8","kp9","kp.","kp/","kp*",
"kp-","kp+","kpenter","kp="}
local textkeystab = {"0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "a", "b", "c", "d", "e", "f", "g", "h", "i", "j", "k",
 "l", "m", "n", "o", "p", "q", "r", "s", "t", "u", "v", "w", "x", "y", "z", " "}
local iskeytab = {}
for i,key in pairs (textkeystab) do
	iskeytab[key] = true
end
-- more cleer names
local friendlyname = {["return"] = " (enter)",[" "] = " (space)", ["kp0"] = " (num 0)",["kp1"] = " (num 1)",["kp2"] = " (num 2)",["kp3"] = " (num 3)",["kp4"] = " (num 4)",
["kp5"] = " (num 5)",["kp6"] = " (num 6)",["kp7"] = " (num 7)",["kp8"] = " (num 8)",["kp9"] = " (num 9)",["kp."] = " (num ,)",["kp/"] = " (num /)",["kp*"] = " (num *)",
["kp-"] = " (num -)",["kp+"] = " (num +)",["kpenter"] = " (num enter)",["kp="] = " (num =)",["up"] = " (arrow up)",["down"] = " (arrow down)",
["left"] = " (arrow left)",["right"] = " (arrow right)"}
local iskey = {}
for i,key in pairs (keytab) do
	iskey[key] = true
end


function restoredefaultconf()
	local filestocreate = {"tankform.conf","customtest1.map","customtest2.map","readme(TanksReloaded).txt"}
	for i,f in pairs(filestocreate) do
		local defaultfile = love.filesystem.read(f) 
		if f == "tankform.conf" then 

			local newfile = io.open(directory.."/tank.conf", "w")
			local defaultconffunc = love.filesystem.load("tankform.conf")
			local defaultconf = defaultconffunc()
			local maplistbackupfunc = loadfile(savedir.."/maplist.bck")
			local maplistbackup = maplistbackupfunc()
			defaultconf.maps = maplistbackup
			local savetext = "return "..table.tostring(defaultconf)
			savetext = string.gsub (savetext,"{","{\n") 
			savetext = string.gsub (savetext,",",",\n")
			savetext = string.gsub (savetext,"}","\n}")
			savetext = string.gsub (savetext,"return {","return {-- acceptable keys: \"0\", \"1\", \"2\", \"3\", \"4\", \5\", \"6\", \"7\", \"8\", \"9\", \"a\", \"b\", \"c\", \"d\", \"e\", \"f\", \"g\", \"h\", \"i\", \"j\", \"k\",\"l\", \"m\", \"n\", \"o\", \"p\", \"q\", \"r\", \"s\", \"t\", \"u\", \"v\", \"w\", \"x\", \"y\", \"z\",\"F1\", \"F2\", \"F3\", \"F4\", \"F5\",\"F6\", \"F7\", \"F8\", \"F9\", \"F10\", \"F11\", \"F12\", \"backspace\", \"tab\", \"lalt\", \"ralt\", \"return\", \" \", \"pageup\", \"pagedown\", \"end\", \"home\",\"insert\", \"delete\", \"lshift\", \"rshift\", \"lctrl\", \"rctrl\", \"[\", \"]\", \"pause\", \"escape\", \"capslock\", \"scroll\", \";\", \",\", \"-\", \".\", \"/\",\"#\", \"\\\", \"=\",\"up\", \"down\", \"left\", \"right\", \"kp0\",\"kp1\",\"kp2\",\"kp3\",\"kp4\",\"kp5\",\"kp6\",\"kp7\",\"kp8\",\"kp9\",\"kp.\",\"kp/\",\"kp*\",\"kp-\",\"kp+\",\"kpenter\",\"kp=\"")
			newfile:write(savetext)
			newfile:close()
		else
			local newfile = io.open(directory.."/"..f, "w")
			newfile:write(defaultfile)
			newfile:close()
		end
	end
end

function assignkey (key,war,tabl) -- loading keys from tank.conf file
	local tab = {tabl}
	local n = 1
	if tabl == "all" then
		n = 4
		tab = {mm0,mp1,mp2,me0}
	end
	
	for i=1,n do -- deftablst
		if iskey[result[deftablst[tab[i]]][key]] then 
			tab[i][war] = result[deftablst[tab[i]]][key]
		else
			table.insert(confprocesstab,#confprocesstab + 1,"Invalid key indicator: \""..result[deftablst[tab[i]]][key].."\"")
		end
	end
end

function soundon()
	playsound = true
	playsounddisplay = "ON"
	soushoot = love.audio.newSource( "shoot.mp3", "static")
	botsoushoot = love.audio.newSource( "shoot.mp3", "static")
	bulletsouhit = love.audio.newSource( "hit.mp3", "static")
	souhit = love.audio.newSource( "hit.mp3", "static")
	souboom = love.audio.newSource( "boom.mp3", "static")
	souride = love.audio.newSource( "ride.mp3", "static")
	botsoushoot:setPitch(0.5)
	botsoushoot:setVolume(0.2)
	bulletsouhit:setPitch(0.38)
	bulletsouhit:setVolume(0.2)
	souride:setVolume(0.3)
	soushoot:setVolume(0.3)
end

function soundoff()
	playsound = false
	playsounddisplay = "OFF"
	soushoot = nil
	botsoushoot = nil
	bulletsouhit = nil
	souhit = nil
	souboom = nil
	souride = nil
end

function reloadmenu()
--botleft = storebotleft
allmenu.resumeeditortab = {
	{dysplay = "MAP EDITOR:"},
	{[mm0.action] = function()
		startmapeditor("resume") -- new resume map
	end,
	dysplay = "RESUME EDITING",
	desc = "Press: "..mm0.action..(friendlyname[mm0.action] or "").." - to load last edited map"},
	{[mm0.action] = function()
		curmenu = allmenu.starteditortab
		menustate = 2
	end,
	dysplay = "START NEW",
	desc = "Press: "..mm0.action..(friendlyname[mm0.action] or "").." - to select or create new map"},
	{[mm0.action] = function()
		curmenu = allmenu.menutab
		menustate = 2
	end,
	dysplay = "BACK",
	desc = "Press: "..mm0.action..(friendlyname[mm0.action] or "").." - to return to main menu"}				
}
allmenu.doyouwantoverwritetab = {
	{dysplay = "OVERWRITE THE MAP?"},
	{[mm0.action] = function()
		local savetext = "return "..table.tostring(map)
		savetext = string.gsub (savetext,"{{","{\n{") 
		savetext = string.gsub (savetext,"},",",},\n")
		savetext = string.gsub (savetext,"}}","}\n}")
		local newfile = io.open(directory.."/"..editorcursor.name..".map", "w")
		newfile:write(savetext)
		newfile:close()
		curmenu = allmenu.menutab
		menustate = 2
	end,
	dysplay = "YES",
	desc = "Map with the name you entered already exsists, press: "..mm0.action..(friendlyname[mm0.action] or "").." to overwrite it"},
	{[mm0.action] = function()
		gamestate = 9
	end,
	dysplay = "NO",
	desc = "Map with the name you entered already exsists, press: "..mm0.action..(friendlyname[mm0.action] or "").." to change the name"}				
}
local mapsizex = 19
local mapsizey = 17
allmenu.starteditortab = {
	{dysplay = "MAP EDITOR:"},
	{[mm0.action] = function()
		curmenu = allmenu.selecteditormapsizetab
		menustate = 2
	end,
	dysplay = "NEW MAP",
	desc = "Press: "..mm0.action..(friendlyname[mm0.action] or "").." - to start creating new map"},
	{[mm0.action] = function()
		mapsizex = 13
		mapsizey = 13
		curmenu = allmenu.selecteditormaptab
		menustate = 2
	end,
	dysplay = "LOAD MAP",
	desc = "Press: "..mm0.action..(friendlyname[mm0.action] or "").." - to load existing map for editing"},
	{[mm0.action] = function()
		curmenu = allmenu.menutab
		menustate = 2
	end,
	dysplay = "BACK",
	desc = "Press: "..mm0.action..(friendlyname[mm0.action] or "").." - to return to main menu"}				
}
allmenu.selecteditormaptab = {
	{dysplay = "SELECT MAP:"},
	{[mm0.action] = function()
		startmapeditor("map",mapid) -- new resume map
	end,
	[mm0.left]= function()
		mapid = mapid - 1
		if mapid < 1 then mapid = #maps end
		setmapdrawtext(mapid)
		curmenu[menustate].dysplay = "MAP: "..mapdraw
	end,
	[mm0.right]= function()
		mapid = mapid + 1
	   	if mapid > #maps then mapid = 1 end
	   	setmapdrawtext(mapid)
		curmenu[menustate].dysplay = "MAP: "..mapdraw
	end,
	dysplay = "MAP: "..mapdraw,
	desc = "Select map using: "..mm0.left..(friendlyname[mm0.left] or "").." and "..mm0.right..(friendlyname[mm0.right] or "").." press: "..mm0.action..(friendlyname[mm0.action] or "").." - to start editing"},
	{[mm0.action] = function()
		curmenu = allmenu.starteditortab
		menustate = 2
	end,
	dysplay = "BACK",
	desc = "Press: "..mm0.action..(friendlyname[mm0.action] or "").." - to return to EDITOR menu"}				
}
allmenu.selecteditormapsizetab = {
	{dysplay = "SELECT MAP SIZE:"},
	{[mm0.action]= function()
		nmenu = true
	end,
	[mm0.left]= function()
		local a = 1
		if nmenu then a = 10 end
		mapsizey = mapsizey - a
		if mapsizey < 17 then mapsizey = 17 end
		curmenu[menustate].dysplay = "Map height: "..mapsizey
	end,
	[mm0.right]= function()
		local a = 1
		if nmenu then a = 10 end
		mapsizey = mapsizey + a
		if mapsizey > 100 then mapsizey = 100 end
		curmenu[menustate].dysplay = "Map height: "..mapsizey
	end,
	botn = botn,
	dysplay = "Map height: "..mapsizey,
	desc = "Set amount using: "..mm0.left..(friendlyname[mm0.left] or "").." and "..mm0.right..(friendlyname[mm0.right] or "").."; Boost x10 holding: "..mm0.action..(friendlyname[mm0.action] or "")},
	{[mm0.action]= function()
		nmenu = true
	end,
	[mm0.left]= function()
		local a = 1
		if nmenu then a = 10 end
		mapsizex = mapsizex - a
		if mapsizex < 19 then mapsizex = 19 end
		curmenu[menustate].dysplay = "Map width: "..mapsizex
	end,
	[mm0.right]= function()
		local a = 1
		if nmenu then a = 10 end
		mapsizex = mapsizex + a
		if mapsizex > 100 then mapsizex = 100 end
		curmenu[menustate].dysplay = "Map width: "..mapsizex
	end,
	botn = botn,
	dysplay = "Map width: "..mapsizex,
	desc = "Set amount using: "..mm0.left..(friendlyname[mm0.left] or "").." and "..mm0.right..(friendlyname[mm0.right] or "").."; Boost x10 holding: "..mm0.action..(friendlyname[mm0.action] or "")},
	{[mm0.action] = function()
		startmapeditor("new", false, mapsizex, mapsizey) -- new resume map
	end,
	dysplay = "CREATE MAP",
	desc = "Press: "..mm0.action..(friendlyname[mm0.action] or "").." - to start editing"},
	{[mm0.action] = function()
		curmenu = allmenu.starteditortab
		menustate = 2
	end,
	dysplay = "BACK",
	desc = "Press: "..mm0.action..(friendlyname[mm0.action] or "").." - to return to EDITOR menu"}				
}
allmenu.restoretab = {
	{dysplay = "SELECT ACTION:"},
	{[mm0.action] = function()
		local defaultfile = io.open(savedir.."/tank.conf","r") 
		local text = defaultfile:read("*a")
		local newfile = io.open("tank.conf", "w")
		newfile:write(text)
		newfile:close()
		defaultfile:close()
		loadconfigfile()
	end, 
	dysplay = "RESTORE LATEST", 
	desc = "Press: "..mm0.action..(friendlyname[mm0.action] or "").." - to restore last correct tank.conf file."},
	{[mm0.action] = function()
		restoredefaultconf()
		loadconfigfile()
	end,
	dysplay = "RESTORE DEDAULT",
	desc = "Press: "..mm0.action..(friendlyname[mm0.action] or "").." - to restore default tank.conf file."},
	{[mm0.action] = function()
		
		local defaultconffunc = loadfile(directory.."/tank.conf")
		local defaultconf = defaultconffunc()
		local maplistbackupfunc = loadfile(savedir.."/maplist.bck")
		local maplistbackup = maplistbackupfunc()
		for i,amap in pairs(maplistbackup) do
			ok, chunk = pcall( loadfile, directory.."/"..amap)
			if ok and not chunk then
				maplistbackup[i] = nil
			end
		end
		defaultconf.maps = maplistbackup
		local savetext = "return "..table.tostring(defaultconf)
		savetext = string.gsub (savetext,"{","{\n") 
		savetext = string.gsub (savetext,",",",\n")
		savetext = string.gsub (savetext,"}","\n}")
		savetext = string.gsub (savetext,"return {","return {-- acceptable keys: \"0\", \"1\", \"2\", \"3\", \"4\", \5\", \"6\", \"7\", \"8\", \"9\", \"a\", \"b\", \"c\", \"d\", \"e\", \"f\", \"g\", \"h\", \"i\", \"j\", \"k\",\"l\", \"m\", \"n\", \"o\", \"p\", \"q\", \"r\", \"s\", \"t\", \"u\", \"v\", \"w\", \"x\", \"y\", \"z\",\"F1\", \"F2\", \"F3\", \"F4\", \"F5\",\"F6\", \"F7\", \"F8\", \"F9\", \"F10\", \"F11\", \"F12\", \"backspace\", \"tab\", \"lalt\", \"ralt\", \"return\", \" \", \"pageup\", \"pagedown\", \"end\", \"home\",\"insert\", \"delete\", \"lshift\", \"rshift\", \"lctrl\", \"rctrl\", \"[\", \"]\", \"pause\", \"escape\", \"capslock\", \"scroll\", \";\", \",\", \"-\", \".\", \"/\",\"#\", \"\\\", \"=\",\"up\", \"down\", \"left\", \"right\", \"kp0\",\"kp1\",\"kp2\",\"kp3\",\"kp4\",\"kp5\",\"kp6\",\"kp7\",\"kp8\",\"kp9\",\"kp.\",\"kp/\",\"kp*\",\"kp-\",\"kp+\",\"kpenter\",\"kp=\"")
		local newfile = io.open(directory.."/tank.conf", "w")
		newfile:write(savetext)
		newfile:close()
		loadconfigfile()
	end,
	dysplay = "CLEANUP MAPS",
	desc = "Press: "..mm0.action..(friendlyname[mm0.action] or "").." - to remove not existing maps from tank.conf file."},
	{[mm0.action]= function()
		local sctualconfigfunc = loadfile(savedir.."/tank.conf")
		actualconfig = sctualconfigfunc()
		
		startmenu()	
	end,
	dysplay = "CONTINUE",
	desc = "Press: "..mm0.action..(friendlyname[mm0.action] or "").." - to lunch menu without restoring tank.conf file."} 
}
allmenu.savequiteditortab = {
	{dysplay = "MAP EDITOR:"},
	{[mm0.action] = function()
		gamestate = 8
	end, 
	dysplay = "RESUME EDITING", 
	desc = "Press: "..mm0.action..(friendlyname[mm0.action] or "").." - to resume editing."},
	{[mm0.action] = function()
		curmenu = allmenu.testeditortab
		menustate = 2
	end,
	dysplay = "TEST MAP",
	desc = "Press: "..mm0.action..(friendlyname[mm0.action] or "").." - to test the map."},
	{[mm0.action] = function()
		gamestate = 9
	end,
	dysplay = "SAVE MAP",
	desc = "Press: "..mm0.action..(friendlyname[mm0.action] or "").." - to save the map to file."},
	{[mm0.action]= function()
		curmenu = allmenu.menutab
		menustate = 2	
	end,
	dysplay = "EXIT EDITOR",
	desc = "Press: "..mm0.action..(friendlyname[mm0.action] or "").." - to return to main menu."} 
}
local testlvl = 0
local testhp = 1
allmenu.testeditortab = {
	{dysplay = "TEST OPTIONS:"},
	{[mm0.action] = function()
		startgame("test",storebotleft,testlvl,testhp)
	end, 
	dysplay = "START TEST", 
	desc = "Press: "..mm0.action..(friendlyname[mm0.action] or "").." - to test the map."},
	{[mm0.action]= function()
		nmenu = true
	end,
	[mm0.left]= function()
		local a = 1
		if nmenu then a = 10 end
		botn = botn - a
		if botn < 1 then botn = 1 end
		curmenu[menustate].dysplay = "MAX BOTS on map: "..botn
		curmenu[menustate].botn = botn
	end,
	[mm0.right]= function()
		local a = 1
		if nmenu then a = 10 end
		botn = botn + a
		curmenu[menustate].dysplay = "MAX BOTS on map: "..botn
		curmenu[menustate].botn = botn
	end,
	botn = botn,
	dysplay = "MAX BOTS on map: "..botn,
	desc = "Set amount using: "..mm0.left..(friendlyname[mm0.left] or "").." and "..mm0.right..(friendlyname[mm0.right] or "").."; Boost x10 holding: "..mm0.action..(friendlyname[mm0.action] or "")},
	{[mm0.action]= function()
		nmenu = true
	end,
	[mm0.left]= function()
		local a = 1
		if nmenu then a = 10 end
		storebotleft = storebotleft - a
		if storebotleft < 1 then storebotleft = 1 end
		curmenu[menustate].dysplay = "Starting amount of BOTS: "..storebotleft
		--curmenu[menustate].botleft = botleft
	end,
	[mm0.right]= function()
		local a = 1
		if nmenu then a = 10 end
		storebotleft = storebotleft + a
		curmenu[menustate].dysplay = "Starting amount of BOTS: "..storebotleft
		--curmenu[menustate].botleft = botleft
	end,
	--botleft = botleft,
	dysplay = "Starting amount of BOTS: "..storebotleft,
	desc = "Set amount using: "..mm0.left..(friendlyname[mm0.left] or "").." and "..mm0.right..(friendlyname[mm0.right] or "").."; Boost x10 holding: "..mm0.action..(friendlyname[mm0.action] or "")},
	
	{[mm0.action]= function()
		nmenu = true
	end,
	[mm0.left]= function()
		local a = 1
		if nmenu then a = 10 end
		testlvl = testlvl - a
		if testlvl < 0 then testlvl = 0 end
		curmenu[menustate].dysplay = "Player level: "..testlvl
		curmenu[menustate].testlvl = testlvl
	end,
	[mm0.right]= function()
		local a = 1
		if nmenu then a = 10 end
		testlvl = testlvl + a
		if testlvl > 99 then testlvl = 99 end
		curmenu[menustate].dysplay = "Player level: "..testlvl
		curmenu[menustate].testlvl = testlvl
	end,
	testlvl = testlvl,
	dysplay = "Player level: "..testlvl,
	desc = "Set amount using: "..mm0.left..(friendlyname[mm0.left] or "").." and "..mm0.right..(friendlyname[mm0.right] or "").."; Boost x10 holding: "..mm0.action..(friendlyname[mm0.action] or "")},
	
	{[mm0.action]= function()
		nmenu = true
	end,
	[mm0.left]= function()
		local a = 1
		if nmenu then a = 10 end
		testhp = testhp - a
		if testhp < 1 then testhp = 1 end
		curmenu[menustate].dysplay = "Player HP: "..testhp
		curmenu[menustate].testhp = testhp
	end,
	[mm0.right]= function()
		local a = 1
		if nmenu then a = 10 end
		testhp = testhp + a
		if testhp > 99 then testhp = 99 end
		curmenu[menustate].dysplay = "Player HP: "..testhp
		curmenu[menustate].testhp = testhp
	end,
	testlvl = testhp,
	dysplay = "Player HP: "..testhp,
	desc = "Set amount using: "..mm0.left..(friendlyname[mm0.left] or "").." and "..mm0.right..(friendlyname[mm0.right] or "").."; Boost x10 holding: "..mm0.action..(friendlyname[mm0.action] or "")},
	{[mm0.left]= function()
		gamemodestate = gamemodestate - 1
		if gamemodestate < 1 then gamemodestate = #gamemodelst end
		curmenu[menustate].dysplay = "GAMEMODE: "..gamemodelst[gamemodestate]
	end,
	[mm0.right]= function()
		gamemodestate = gamemodestate + 1
	   	if gamemodestate > #gamemodelst then gamemodestate  = 1 end
		curmenu[menustate].dysplay = "GAMEMODE: "..gamemodelst[gamemodestate]
	end,
	dysplay = "GAMEMODE: "..gamemodelst[gamemodestate],
	desc = "Select game mode using: "..mm0.left..(friendlyname[mm0.left] or "").." and "..mm0.right..(friendlyname[mm0.right] or "")},
	{[mm0.action]= function()
		nmenu = true
	end,
	[mm0.left]= function()
		local a = 1
		if nmenu then a = 10 end
		playerbasehp = playerbasehp - a
		if playerbasehp < 1 then playerbasehp = 1 end
		curmenu[menustate].dysplay = "PLAYER BASE HP: "..playerbasehp
	end,
	[mm0.right]= function()
		local a = 1
		if nmenu then a = 10 end
		playerbasehp = playerbasehp + a
		curmenu[menustate].dysplay = "PLAYER BASE HP: "..playerbasehp
	end,
	dysplay = "PLAYER BASE HP: "..playerbasehp,
	desc = "Set amount using: "..mm0.left..(friendlyname[mm0.left] or "").." and "..mm0.right..(friendlyname[mm0.right] or "").."; Boost x10 holding: "..mm0.action..(friendlyname[mm0.action] or "")},
	{[mm0.action]= function()
		nmenu = true
	end,
	[mm0.left]= function()
		local a = 1
		if nmenu then a = 10 end
		botbasehp = botbasehp - a
		if botbasehp < 1 then botbasehp = 1 end
		curmenu[menustate].dysplay = "BOT BASE HP: player lvl + "..botbasehp
	end,
	[mm0.right]= function()
		local a = 1
		if nmenu then a = 10 end
		botbasehp = botbasehp + a
		curmenu[menustate].dysplay = "BOT BASE HP: player lvl + "..botbasehp
	end,
	dysplay = "BOT BASE HP: player lvl + "..botbasehp,
	desc = "Set amount using: "..mm0.left..(friendlyname[mm0.left] or "").." and "..mm0.right..(friendlyname[mm0.right] or "").."; Boost x10 holding: "..mm0.action..(friendlyname[mm0.action] or "")},
	{[mm0.action]= function()
		curmenu = allmenu.savequiteditortab
		menustate = 2	
	end,
	dysplay = "BACK",
	desc = "Press: "..mm0.action..(friendlyname[mm0.action] or "").." - to return to editor menu."} 
}
allmenu.suboptions = {
	{dysplay = "OPTIONS:"},
	{[mm0.action] = function()
		curmenu = allmenu.dificulitytab
		menustate = 2
	end,
	dysplay = "DIFFICULITY",
	desc = "Press: "..mm0.action..(friendlyname[mm0.action] or "").." - to open difficulty menu"},
	{[mm0.action] = function()
		if playsound then
			soundoff()
			curmenu[menustate].dysplay = "SOUND: "..playsounddisplay
		else
			soundon()
			curmenu[menustate].dysplay = "SOUND: "..playsounddisplay
		end
	end,
	dysplay = "SOUND: "..playsounddisplay,
	desc = "Press: "..mm0.action..(friendlyname[mm0.action] or "").." - to toggle sound"},
	{[mm0.action] = function()
		curmenu = allmenu.keysettingtab
		menustate = 2
	end,
	dysplay = "KEY SETTINGS",
	desc = "Press: "..mm0.action..(friendlyname[mm0.action] or "").." - to change the key settings"},
	{[mm0.action] = function()
		actualconfig.difficulity_settings.small_map_size = smallmapsize
		actualconfig.difficulity_settings.max_bots_on_small_map = slallbotn
		actualconfig.difficulity_settings.medium_map_size = mediummapsize
		actualconfig.difficulity_settings.max_bots_on_medium_map = mediumbotn
		actualconfig.difficulity_settings.large_map_size = largemapsize
		actualconfig.difficulity_settings.max_bots_on_large_map = largebotn
		actualconfig.difficulity_settings.max_bots_on_very_large_map = verylalgebotn
		actualconfig.difficulity_settings.starting_amount_of_bots = storebotleft
		actualconfig.difficulity_settings.new_bots_per_map = botincreeseamount
		actualconfig.difficulity_settings.player_base_hp = playerbasehp
		actualconfig.difficulity_settings.bot_base_hp = botbasehp
		if bulletandisplay == "ON" then 
			actualconfig.difficulity_settings.bullet_anihilation = 1
		else
			actualconfig.difficulity_settings.bullet_anihilation = 0
		end
		if firedetectordisplay == "ON" then 
			actualconfig.difficulity_settings.fire_detector = 1
		else
			actualconfig.difficulity_settings.fire_detector = 0
		end
		if playsound then
			actualconfig.sound = 1
		else
			actualconfig.sound = 0
		end
		actualconfig.menu_keys = mm0
		actualconfig.player1_keys = mp1
		actualconfig.player2_keys = mp2
		actualconfig.editor_keys = me0
		local savetext = "return "..table.tostring(actualconfig)
		savetext = string.gsub (savetext,"{","{\n") 
		savetext = string.gsub (savetext,",",",\n")
		savetext = string.gsub (savetext,"}","\n}")
		savetext = string.gsub (savetext,"return {","return {-- acceptable keys: \"0\", \"1\", \"2\", \"3\", \"4\", \5\", \"6\", \"7\", \"8\", \"9\", \"a\", \"b\", \"c\", \"d\", \"e\", \"f\", \"g\", \"h\", \"i\", \"j\", \"k\",\"l\", \"m\", \"n\", \"o\", \"p\", \"q\", \"r\", \"s\", \"t\", \"u\", \"v\", \"w\", \"x\", \"y\", \"z\",\"F1\", \"F2\", \"F3\", \"F4\", \"F5\",\"F6\", \"F7\", \"F8\", \"F9\", \"F10\", \"F11\", \"F12\", \"backspace\", \"tab\", \"lalt\", \"ralt\", \"return\", \" \", \"pageup\", \"pagedown\", \"end\", \"home\",\"insert\", \"delete\", \"lshift\", \"rshift\", \"lctrl\", \"rctrl\", \"[\", \"]\", \"pause\", \"escape\", \"capslock\", \"scroll\", \";\", \",\", \"-\", \".\", \"/\",\"#\", \"\\\", \"=\",\"up\", \"down\", \"left\", \"right\", \"kp0\",\"kp1\",\"kp2\",\"kp3\",\"kp4\",\"kp5\",\"kp6\",\"kp7\",\"kp8\",\"kp9\",\"kp.\",\"kp/\",\"kp*\",\"kp-\",\"kp+\",\"kpenter\",\"kp=\"")
		local newfile = io.open(directory.."/tank.conf", "w")
		newfile:write(savetext)
		newfile:close()
		loadconfigfile()
	end,
	dysplay = "SAVE SETTINGS",
	desc = "Press: "..mm0.action..(friendlyname[mm0.action] or "").." - to save the current settings"},
	{[mm0.action] = function()
		restoredefaultconf()
		loadconfigfile()
	end,
	dysplay = "RESTORE DEFAULT",
	desc = "Press: "..mm0.action..(friendlyname[mm0.action] or "").." - to return default settings"},			
	{[mm0.action] = function()
		curmenu = allmenu.menutab
		menustate = 2
	end,
	dysplay = "BACK",
	desc = "Press: "..mm0.action..(friendlyname[mm0.action] or "").." - to return to main menu"}				
	}

allmenu.dificulitybotntab = {
	{dysplay = "MAX BOTS on map:"},
	{[mm0.action]= function()
		nmenu = true
	end,
	[mm0.left]= function()
		local a = 10
		if nmenu then a = 100 end
		smallmapsize = smallmapsize - a
		if smallmapsize < 10 then smallmapsize = 10 end
		curmenu[menustate].dysplay = "SMALL MAP SIZE: "..smallmapsize
		--curmenu[menustate].botn = botn
	end,
	[mm0.right]= function()
		local a = 10
		if nmenu then a = 100 end
		smallmapsize = smallmapsize + a
		if smallmapsize > mediummapsize then smallmapsize = mediummapsize - 10 end
		curmenu[menustate].dysplay = "SMALL MAP SIZE: "..smallmapsize
		--curmenu[menustate].botn = botn
	end,
	botn = botn,
	dysplay = "SMALL MAP SIZE: "..smallmapsize,
	desc = "Set amount using: "..mm0.left..(friendlyname[mm0.left] or "").." and "..mm0.right..(friendlyname[mm0.right] or "").."; Boost x10 holding: "..mm0.action..(friendlyname[mm0.action] or "")},
	{[mm0.action]= function()
		nmenu = true
	end,
	[mm0.left]= function()
		local a = 1
		if nmenu then a = 10 end
		slallbotn = slallbotn - a
		if slallbotn < 1 then slallbotn = 1 end
		curmenu[menustate].dysplay = "Bots on small map: "..slallbotn
		--curmenu[menustate].botn = botn
	end,
	[mm0.right]= function()
		local a = 1
		if nmenu then a = 10 end
		slallbotn = slallbotn + a
		curmenu[menustate].dysplay = "Bots on small map: "..slallbotn
		--curmenu[menustate].botn = botn
	end,
	botn = botn,
	dysplay = "Bots on small map: "..slallbotn,
	desc = "Set amount using: "..mm0.left..(friendlyname[mm0.left] or "").." and "..mm0.right..(friendlyname[mm0.right] or "").."; Boost x10 holding: "..mm0.action..(friendlyname[mm0.action] or "")},
	{[mm0.action]= function()
		nmenu = true
	end,
	[mm0.left]= function()
		local a = 10
		if nmenu then a = 100 end
		mediummapsize = mediummapsize - a
		if mediummapsize < smallmapsize then mediummapsize = smallmapsize + 10 end
		curmenu[menustate].dysplay = "MEDIUM MAP SIZE: "..mediummapsize
		--curmenu[menustate].botn = botn
	end,
	[mm0.right]= function()
		local a = 10
		if nmenu then a = 100 end
		mediummapsize = mediummapsize + a
		if mediummapsize > largemapsize then mediummapsize = largemapsize - 10 end
		curmenu[menustate].dysplay = "MEDIUM MAP SIZE: "..mediummapsize
		--curmenu[menustate].botn = botn
	end,
	botn = botn,
	dysplay = "MEDIUM MAP SIZE: "..mediummapsize,
	desc = "Set amount using: "..mm0.left..(friendlyname[mm0.left] or "").." and "..mm0.right..(friendlyname[mm0.right] or "").."; Boost x10 holding: "..mm0.action..(friendlyname[mm0.action] or "")},
	{[mm0.action]= function()
		nmenu = true
	end,
	[mm0.left]= function()
		local a = 1
		if nmenu then a = 10 end
		mediumbotn = mediumbotn - a
		if mediumbotn < 1 then mediumbotn = 1 end
		curmenu[menustate].dysplay = "Bots on medium map: "..mediumbotn
		--curmenu[menustate].botn = botn
	end,
	[mm0.right]= function()
		local a = 1
		if nmenu then a = 10 end
		mediumbotn = mediumbotn + a
		curmenu[menustate].dysplay = "Bots on medium map: "..mediumbotn
		--curmenu[menustate].botn = botn
	end,
	botn = botn,
	dysplay = "Bots on medium map: "..mediumbotn,
	desc = "Set amount using: "..mm0.left..(friendlyname[mm0.left] or "").." and "..mm0.right..(friendlyname[mm0.right] or "").."; Boost x10 holding: "..mm0.action..(friendlyname[mm0.action] or "")},
	{[mm0.action]= function()
		nmenu = true
	end,
	[mm0.left]= function()
		local a = 10
		if nmenu then a = 100 end
		largemapsize = largemapsize - a
		if largemapsize < mediummapsize then largemapsize = mediummapsize + 10 end
		curmenu[menustate].dysplay = "LARGE MAP SIZE: "..largemapsize
		--curmenu[menustate].botn = botn
	end,
	[mm0.right]= function()
		local a = 10
		if nmenu then a = 100 end
		largemapsize = largemapsize + a
		if largemapsize > 10000 then largemapsize = 10000 - 10 end
		curmenu[menustate].dysplay = "LARGE MAP SIZE: "..largemapsize
		--curmenu[menustate].botn = botn
	end,
	botn = botn,
	dysplay = "LARGE MAP SIZE: "..largemapsize,
	desc = "Set amount using: "..mm0.left..(friendlyname[mm0.left] or "").." and "..mm0.right..(friendlyname[mm0.right] or "").."; Boost x10 holding: "..mm0.action..(friendlyname[mm0.action] or "")},
	{[mm0.action]= function()
		nmenu = true
	end,
	[mm0.left]= function()
		local a = 1
		if nmenu then a = 10 end
		largebotn = largebotn - a
		if largebotn < 1 then largebotn = 1 end
		curmenu[menustate].dysplay = "Bots on large map: "..largebotn
		--curmenu[menustate].botn = botn
	end,
	[mm0.right]= function()
		local a = 1
		if nmenu then a = 10 end
		largebotn = largebotn + a
		curmenu[menustate].dysplay = "Bots on large map: "..largebotn
		--curmenu[menustate].botn = botn
	end,
	botn = botn,
	dysplay = "Bots on large map: "..largebotn,
	desc = "Set amount using: "..mm0.left..(friendlyname[mm0.left] or "").." and "..mm0.right..(friendlyname[mm0.right] or "").."; Boost x10 holding: "..mm0.action..(friendlyname[mm0.action] or "")},
		{[mm0.action]= function()
		nmenu = true
	end,
	[mm0.left]= function()
		local a = 1
		if nmenu then a = 10 end
		verylalgebotn = verylalgebotn - a
		if verylalgebotn < 1 then verylalgebotn = 1 end
		curmenu[menustate].dysplay = "Bots on very large map: "..verylalgebotn
		--curmenu[menustate].botn = botn
	end,
	[mm0.right]= function()
		local a = 1
		if nmenu then a = 10 end
		verylalgebotn = verylalgebotn + a
		curmenu[menustate].dysplay = "Bots on very large map: "..verylalgebotn
		--curmenu[menustate].botn = botn
	end,
	botn = botn,
	dysplay = "Bots on very large map: "..verylalgebotn,
	desc = "Set amount using: "..mm0.left..(friendlyname[mm0.left] or "").." and "..mm0.right..(friendlyname[mm0.right] or "").."; Boost x10 holding: "..mm0.action..(friendlyname[mm0.action] or "")},
	{[mm0.action] = function()
		curmenu = allmenu.dificulitytab
		menustate = 2
	end,
	dysplay = "BACK",
	desc = "Press: "..mm0.action..(friendlyname[mm0.action] or "").." - to return to options menu"}	
}

allmenu.dificulitytab = {
	{dysplay = "DIFFICULITY:"},
	{[mm0.action]= function()
		curmenu = allmenu.dificulitybotntab
		menustate = 2
	end,
	botn = botn,
	dysplay = "MAX BOTS on map",
	desc = "Press: "..mm0.action..(friendlyname[mm0.action] or "").."to set the max amout of bots on maps of any size"},
	{[mm0.action]= function()
		nmenu = true
	end,
	[mm0.left]= function()
		local a = 1
		if nmenu then a = 10 end
		storebotleft = storebotleft - a
		if storebotleft < 1 then storebotleft = 1 end
		curmenu[menustate].dysplay = "Starting amount of BOTS: "..storebotleft
		--curmenu[menustate].botleft = botleft
	end,
	[mm0.right]= function()
		local a = 1
		if nmenu then a = 10 end
		storebotleft = storebotleft + a
		curmenu[menustate].dysplay = "Starting amount of BOTS: "..storebotleft
		--curmenu[menustate].botleft = storebotleft
	end,
	storebotleft = storebotleft,
	dysplay = "Starting amount of BOTS: "..storebotleft,
	desc = "Set amount using: "..mm0.left..(friendlyname[mm0.left] or "").." and "..mm0.right..(friendlyname[mm0.right] or "").."; Boost x10 holding: "..mm0.action..(friendlyname[mm0.action] or "")},
	{[mm0.action]= function()
		nmenu = true
	end,
	[mm0.left]= function()
		local a = 1
		if nmenu then a = 10 end
		botincreeseamount = botincreeseamount - a
		if botincreeseamount < 1 then botincreeseamount = 1 end
		curmenu[menustate].dysplay = "New BOTS per map: "..botincreeseamount
		curmenu[menustate].botincreeseamount = botincreeseamount
	end,
	[mm0.right]= function()
		local a = 1
		if nmenu then a = 10 end
		botincreeseamount = botincreeseamount + a
		curmenu[menustate].dysplay = "New BOTS per map: "..botincreeseamount
		curmenu[menustate].botincreeseamount = botincreeseamount
	end,
	botincreeseamount = botincreeseamount,
	dysplay = "New BOTS per map: "..botincreeseamount,
	desc = "Set amount using: "..mm0.left..(friendlyname[mm0.left] or "").." and "..mm0.right..(friendlyname[mm0.right] or "").."; Boost x10 holding: "..mm0.action..(friendlyname[mm0.action] or "")},
	{[mm0.action] = function()
		if bulletandisplay == "ON" then
			bulletandisplay = "OFF"
			curmenu[menustate].dysplay = "BULLET ANNIHILATION: "..bulletandisplay
		else
			bulletandisplay = "ON"
			curmenu[menustate].dysplay = "BULLET ANNIHILATION: "..bulletandisplay
		end
	end,
	dysplay = "BULLET ANNIHILATION: "..bulletandisplay,
	desc = "Press: "..mm0.action..(friendlyname[mm0.action] or "").." - to toggle if bot bullets should destroy player bullets"},
	{[mm0.action] = function()
		if firedetectordisplay == "ON" then
			firedetectordisplay = "OFF"
			curmenu[menustate].dysplay = "FIRE DETECTOR: "..firedetectordisplay
		else
			firedetectordisplay = "ON"
			curmenu[menustate].dysplay = "FIRE DETECTOR: "..firedetectordisplay
		end
	end,
	dysplay = "FIRE DETECTOR: "..firedetectordisplay,
	desc = "Press: "..mm0.action..(friendlyname[mm0.action] or "").." - to toggle if fire and lava should have colision if the player has low HP"},
	{[mm0.action]= function()
		nmenu = true
	end,
	[mm0.left]= function()
		local a = 1
		if nmenu then a = 10 end
		playerbasehp = playerbasehp - a
		if playerbasehp < 1 then playerbasehp = 1 end
		curmenu[menustate].dysplay = "PLAYER BASE HP: "..playerbasehp
	end,
	[mm0.right]= function()
		local a = 1
		if nmenu then a = 10 end
		playerbasehp = playerbasehp + a
		curmenu[menustate].dysplay = "PLAYER BASE HP: "..playerbasehp
	end,
	dysplay = "PLAYER BASE HP: "..playerbasehp,
	desc = "Set amount using: "..mm0.left..(friendlyname[mm0.left] or "").." and "..mm0.right..(friendlyname[mm0.right] or "").."; Boost x10 holding: "..mm0.action..(friendlyname[mm0.action] or "")},
	{[mm0.action]= function()
		nmenu = true
	end,
	[mm0.left]= function()
		local a = 1
		if nmenu then a = 10 end
		botbasehp = botbasehp - a
		if botbasehp < 1 then botbasehp = 1 end
		curmenu[menustate].dysplay = "BOT BASE HP: player lvl + "..botbasehp
	end,
	[mm0.right]= function()
		local a = 1
		if nmenu then a = 10 end
		botbasehp = botbasehp + a
		curmenu[menustate].dysplay = "BOT BASE HP: player lvl + "..botbasehp
	end,
	dysplay = "BOT BASE HP: player lvl + "..botbasehp,
	desc = "Set amount using: "..mm0.left..(friendlyname[mm0.left] or "").." and "..mm0.right..(friendlyname[mm0.right] or "").."; Boost x10 holding: "..mm0.action..(friendlyname[mm0.action] or "")},
	{[mm0.action] = function()
		curmenu = allmenu.suboptions
		menustate = 2
	end,
	dysplay = "BACK",
	desc = "Press: "..mm0.action..(friendlyname[mm0.action] or "").." - to return to options menu"}				
}
allmenu.startgametab = {
	{dysplay = "START GAME:"},
	{[mm0.action] = function()
		botincreese = botincreeseamount
		startgame("new",storebotleft)
	end,
	dysplay = "NUMBER OF PLAYERS: 1",
	desc = "Press: "..mm0.action..(friendlyname[mm0.action] or "").." - to start the game.\nGame controls: moving: "..mp1.up..(friendlyname[mp1.up] or "").."; "..mp1.down..(friendlyname[mp1.down] or "")..";"..mp1.left..(friendlyname[mp1.left] or "").."; "..mp1.right..(friendlyname[mp1.right] or "")..";\nshoting - "..mp1.shoot..(friendlyname[mp1.shoot] or "").." (You may change it in tank.conf)"},
	{[mm0.action] = function()
		botincreese = botincreeseamount
		startgame("new",storebotleft)
	end,
	[mm0.left]= function()
		mapid = mapid - 1
		if mapid < 1 then mapid = #maps end
		setmapdrawtext(mapid)
		curmenu[menustate].dysplay = "MAP: "..mapdraw
	end,
	[mm0.right]= function()
		mapid = mapid + 1
	   	if mapid > #maps then mapid = 1 end
	   	setmapdrawtext(mapid)
		curmenu[menustate].dysplay = "MAP: "..mapdraw
	end,
	dysplay = "MAP: "..mapdraw,
	desc = "Select map using: "..mm0.left..(friendlyname[mm0.left] or "").." and "..mm0.right..(friendlyname[mm0.right] or "").." press: "..mm0.action..(friendlyname[mm0.action] or "").." - to start the game\nGame controls: moving: "..mp1.up..(friendlyname[mp1.up] or "").."; "..mp1.down..(friendlyname[mp1.down] or "")..";"..mp1.left..(friendlyname[mp1.left] or "").."; "..mp1.right..(friendlyname[mp1.right] or "")..";\nshoting - "..mp1.shoot..(friendlyname[mp1.shoot] or "").." (You may change it in tank.conf)"},
	{[mm0.action] = function()
		botincreese = botincreeseamount
		startgame("new",storebotleft)
	end,
	[mm0.left]= function()
		gamemodestate = gamemodestate - 1
		if gamemodestate < 1 then gamemodestate = #gamemodelst end
		curmenu[menustate].dysplay = "GAMEMODE: "..gamemodelst[gamemodestate]
	end,
	[mm0.right]= function()
		gamemodestate = gamemodestate + 1
	   	if gamemodestate > #gamemodelst then gamemodestate  = 1 end
		curmenu[menustate].dysplay = "GAMEMODE: "..gamemodelst[gamemodestate]
	end,
	dysplay = "GAMEMODE: "..gamemodelst[gamemodestate],
	desc = "Select game mode using: "..mm0.left..(friendlyname[mm0.left] or "").." and "..mm0.right..(friendlyname[mm0.right] or "").." press: "..mm0.action..(friendlyname[mm0.action] or "").." - to start the game.\nGame controls: moving: "..mp1.up..(friendlyname[mp1.up] or "").."; "..mp1.down..(friendlyname[mp1.down] or "")..";"..mp1.left..(friendlyname[mp1.left] or "").."; "..mp1.right..(friendlyname[mp1.right] or "")..";\nshoting - "..mp1.shoot..(friendlyname[mp1.shoot] or "").." (You may change it in tank.conf)"},
	{[mm0.action] = function()
		curmenu = allmenu.menutab
		menustate = 2
	end,
	dysplay = "BACK",
	desc = "Press: "..mm0.action..(friendlyname[mm0.action] or "").." - to return to main menu"}
}
allmenu.startresumegametab = {
	{dysplay = "START/RESUME GAME:"},
	{[mm0.action] = function()
		loadgame()
	end,
	dysplay = "RESUME",
	desc = "Press: "..mm0.action..(friendlyname[mm0.action] or "").." - to resume last session"},
	{[mm0.action] = function()
		curmenu = allmenu.startgametab
		menustate = 2
	end,
	dysplay = "START GAME",
	desc = "Press: "..mm0.action..(friendlyname[mm0.action] or "").." - to start the game.\nGame controls: moving: "..mp1.up..(friendlyname[mp1.up] or "").."; "..mp1.down..(friendlyname[mp1.down] or "")..";"..mp1.left..(friendlyname[mp1.left] or "").."; "..mp1.right..(friendlyname[mp1.right] or "")..";\nshoting - "..mp1.shoot..(friendlyname[mp1.shoot] or "").." (You may change it in tank.conf)"},
}

allmenu.player1keysettingtab = {
	{dysplay = "Player 1 KEY SETTINGS:"},
	{[mm0.action] = function()
		readkey = {mp1,"up"}
	end,
	dysplay = "UP KEY: "..mp1.up..(friendlyname[mp1.up] or ""),
	desc = "Press: "..mm0.action..(friendlyname[mm0.action] or "")..", then press the new key"},
	{[mm0.action] = function()
		readkey = {mp1,"down"}
	end,
	dysplay = "DOWN KEY: "..mp1.down..(friendlyname[mp1.down] or ""),
	desc = "Press: "..mm0.action..(friendlyname[mm0.action] or "")..", then press the new key"},
	{[mm0.action] = function()
		readkey = {mp1,"left"}
	end,
	dysplay = "LEFT KEY: "..mp1.left..(friendlyname[mp1.left] or ""),
	desc = "Press: "..mm0.action..(friendlyname[mm0.action] or "")..", then press the new key"},
	{[mm0.action] = function()
		readkey = {mp1,"right"}
	end,
	dysplay = "RIGHT KEY: "..mp1.right..(friendlyname[mp1.right] or ""),
	desc = "Press: "..mm0.action..(friendlyname[mm0.action] or "")..", then press the new key"},
	{[mm0.action] = function()
		readkey = {mp1,"shoot"}
	end,
	dysplay = "SHOOT KEY: "..mp1.shoot..(friendlyname[mp1.shoot] or ""),
	desc = "Press: "..mm0.action..(friendlyname[mm0.action] or "")..", then press the new key"},
	{[mm0.action] = function()
		curmenu = allmenu.keysettingtab
		menustate = 2	
	end,
	dysplay = "BACK",
	desc = "Press: "..mm0.action..(friendlyname[mm0.action] or "").." - to return to key settings menu"},
}

allmenu.menukeysettingtab = {
	{dysplay = "Player 1 KEY SETTINGS:"},
	{[mm0.action] = function()
		readkey = {mm0,"up"}
	end,
	dysplay = "UP KEY: "..mm0.up..(friendlyname[mm0.up] or ""),
	desc = "Press: "..mm0.action..(friendlyname[mm0.action] or "")..", then press the new key"},
	{[mm0.action] = function()
		readkey = {mm0,"down"}
	end,
	dysplay = "DOWN KEY: "..mm0.down..(friendlyname[mm0.down] or ""),
	desc = "Press: "..mm0.action..(friendlyname[mm0.action] or "")..", then press the new key"},
	{[mm0.action] = function()
		readkey = {mm0,"left"}
	end,
	dysplay = "LEFT KEY: "..mm0.left..(friendlyname[mm0.left] or ""),
	desc = "Press: "..mm0.action..(friendlyname[mm0.action] or "")..", then press the new key"},
	{[mm0.action] = function()
		readkey = {mm0,"right"}
	end,
	dysplay = "RIGHT KEY: "..mm0.right..(friendlyname[mm0.right] or ""),
	desc = "Press: "..mm0.action..(friendlyname[mm0.action] or "")..", then press the new key"},
	{[mm0.action] = function()
		readkey = {mm0,"action"}
	end,
	dysplay = "ACTION KEY: "..mm0.action..(friendlyname[mm0.action] or ""),
	desc = "Press: "..mm0.action..(friendlyname[mm0.action] or "")..", then press the new key"},
	{[mm0.action] = function()
		readkey = {mm0,"pause"}
	end,
	dysplay = "PAUSE KEY: "..mm0.pause..(friendlyname[mm0.pause] or ""),
	desc = "Press: "..mm0.action..(friendlyname[mm0.action] or "")..", then press the new key"},
	{[mm0.action] = function()
		curmenu = allmenu.keysettingtab
		menustate = 2	
	end,
	dysplay = "BACK",
	desc = "Press: "..mm0.action..(friendlyname[mm0.action] or "").." - to return to key settings menu"},
}

allmenu.editorkeysettingtab = {
	{dysplay = "MAP EDITOR KEY SETTINGS:"},
	{[mm0.action] = function()
		readkey = {me0,"up"}
	end,
	dysplay = "UP KEY: "..me0.up..(friendlyname[me0.up] or ""),
	desc = "Press: "..mm0.action..(friendlyname[mm0.action] or "")..", then press the new key"},
	{[mm0.action] = function()
		readkey = {me0,"down"}
	end,
	dysplay = "DOWN KEY: "..me0.down..(friendlyname[me0.down] or ""),
	desc = "Press: "..mm0.action..(friendlyname[mm0.action] or "")..", then press the new key"},
	{[mm0.action] = function()
		readkey = {me0,"left"}
	end,
	dysplay = "LEFT KEY: "..me0.left..(friendlyname[me0.left] or ""),
	desc = "Press: "..mm0.action..(friendlyname[mm0.action] or "")..", then press the new key"},
	{[mm0.action] = function()
		readkey = {me0,"right"}
	end,
	dysplay = "RIGHT KEY: "..me0.right..(friendlyname[me0.right] or ""),
	desc = "Press: "..mm0.action..(friendlyname[mm0.action] or "")..", then press the new key"},
	{[mm0.action] = function()
		readkey = {me0,"next"}
	end,
	dysplay = "NEXT KEY: "..me0.next..(friendlyname[me0.next] or ""),
	desc = "Press: "..mm0.action..(friendlyname[mm0.action] or "")..", then press the new key"},
	{[mm0.action] = function()
		readkey = {me0,"previous"}
	end,
	dysplay = "PREVIOUS KEY: "..me0.previous..(friendlyname[me0.previous] or ""),
	desc = "Press: "..mm0.action..(friendlyname[mm0.action] or "")..", then press the new key"}, --previous
	{[mm0.action] = function()
		readkey = {me0,"save_quit"}
	end,
	dysplay = "SAVE_QUIT KEY: "..me0.save_quit..(friendlyname[me0.save_quit] or ""),
	desc = "Press: "..mm0.action..(friendlyname[mm0.action] or "")..", then press the new key"}, --save_quit
	{[mm0.action] = function()
		curmenu = allmenu.keysettingtab
		menustate = 2	
	end,
	dysplay = "BACK",
	desc = "Press: "..mm0.action..(friendlyname[mm0.action] or "").." - to return to key settings menu"},
}

allmenu.keysettingtab = {
	{dysplay = "KEY SETTINGS:"},
	{[mm0.action] = function()
		curmenu = allmenu.player1keysettingtab
		menustate = 2
	end,
	dysplay = "PLAYER 1",
	desc = "Press: "..mm0.action..(friendlyname[mm0.action] or "").." - to set keys for player 1"},
	{[mm0.action] = function()
		
	end,
	dysplay = "PLAYER 2",
	desc = "Press: "..mm0.action..(friendlyname[mm0.action] or "").." - to set keys for player 2"},
	{[mm0.action] = function()
		curmenu = allmenu.menukeysettingtab
		menustate = 2
	end,
	dysplay = "MENU",
	desc = "Press: "..mm0.action..(friendlyname[mm0.action] or "").." - to set keys for menu"},
	{[mm0.action] = function()
		curmenu = allmenu.editorkeysettingtab
		menustate = 2
	end,
	dysplay = "Editor",
	desc = "Press: "..mm0.action..(friendlyname[mm0.action] or "").." - to set keys for editor"},
	{[mm0.action] = function()
		actualconfig.menu_keys = mm0
		actualconfig.player1_keys = mp1
		actualconfig.player2_keys = mp2
		actualconfig.editor_keys = me0
		local savetext = "return "..table.tostring(actualconfig)
		savetext = string.gsub (savetext,"{","{\n") 
		savetext = string.gsub (savetext,",",",\n")
		savetext = string.gsub (savetext,"}","\n}")
		savetext = string.gsub (savetext,"return {","return {-- acceptable keys: \"0\", \"1\", \"2\", \"3\", \"4\", \5\", \"6\", \"7\", \"8\", \"9\", \"a\", \"b\", \"c\", \"d\", \"e\", \"f\", \"g\", \"h\", \"i\", \"j\", \"k\",\"l\", \"m\", \"n\", \"o\", \"p\", \"q\", \"r\", \"s\", \"t\", \"u\", \"v\", \"w\", \"x\", \"y\", \"z\",\"F1\", \"F2\", \"F3\", \"F4\", \"F5\",\"F6\", \"F7\", \"F8\", \"F9\", \"F10\", \"F11\", \"F12\", \"backspace\", \"tab\", \"lalt\", \"ralt\", \"return\", \" \", \"pageup\", \"pagedown\", \"end\", \"home\",\"insert\", \"delete\", \"lshift\", \"rshift\", \"lctrl\", \"rctrl\", \"[\", \"]\", \"pause\", \"escape\", \"capslock\", \"scroll\", \";\", \",\", \"-\", \".\", \"/\",\"#\", \"\\\", \"=\",\"up\", \"down\", \"left\", \"right\", \"kp0\",\"kp1\",\"kp2\",\"kp3\",\"kp4\",\"kp5\",\"kp6\",\"kp7\",\"kp8\",\"kp9\",\"kp.\",\"kp/\",\"kp*\",\"kp-\",\"kp+\",\"kpenter\",\"kp=\"")
		local newfile = io.open(directory.."/tank.conf", "w")
		newfile:write(savetext)
		newfile:close()
		loadconfigfile()
	end,
	dysplay = "SAVE SETTINGS",
	desc = "Press: "..mm0.action..(friendlyname[mm0.action] or "").." - save the key settings"},
	{[mm0.action] = function()
		local backupkeyfunc = love.filesystem.load("keydefault.conf")
		local backupkeys = backupkeyfunc() 
		actualconfig.menu_keys = backupkeys.menu_keys
		actualconfig.player1_keys = backupkeys.player1_keys
		actualconfig.player2_keys = backupkeys.player2_keys
		actualconfig.editor_keys = backupkeys.editor_keys
		local savetext = "return "..table.tostring(actualconfig)
		savetext = string.gsub (savetext,"{","{\n") 
		savetext = string.gsub (savetext,",",",\n")
		savetext = string.gsub (savetext,"}","\n}")
		savetext = string.gsub (savetext,"return {","return {-- acceptable keys: \"0\", \"1\", \"2\", \"3\", \"4\", \5\", \"6\", \"7\", \"8\", \"9\", \"a\", \"b\", \"c\", \"d\", \"e\", \"f\", \"g\", \"h\", \"i\", \"j\", \"k\",\"l\", \"m\", \"n\", \"o\", \"p\", \"q\", \"r\", \"s\", \"t\", \"u\", \"v\", \"w\", \"x\", \"y\", \"z\",\"F1\", \"F2\", \"F3\", \"F4\", \"F5\",\"F6\", \"F7\", \"F8\", \"F9\", \"F10\", \"F11\", \"F12\", \"backspace\", \"tab\", \"lalt\", \"ralt\", \"return\", \" \", \"pageup\", \"pagedown\", \"end\", \"home\",\"insert\", \"delete\", \"lshift\", \"rshift\", \"lctrl\", \"rctrl\", \"[\", \"]\", \"pause\", \"escape\", \"capslock\", \"scroll\", \";\", \",\", \"-\", \".\", \"/\",\"#\", \"\\\", \"=\",\"up\", \"down\", \"left\", \"right\", \"kp0\",\"kp1\",\"kp2\",\"kp3\",\"kp4\",\"kp5\",\"kp6\",\"kp7\",\"kp8\",\"kp9\",\"kp.\",\"kp/\",\"kp*\",\"kp-\",\"kp+\",\"kpenter\",\"kp=\"")
		local newfile = io.open(directory.."/tank.conf", "w")
		newfile:write(savetext)
		newfile:close()
		loadconfigfile()
	end,
	dysplay = "RESTORE DEFAULT",
	desc = "Press: "..mm0.action..(friendlyname[mm0.action] or "").." - restore default keys"},
	{[mm0.action] = function()
		curmenu = allmenu.suboptions
		menustate = 2	
	end,
	dysplay = "BACK",
	desc = "Press: "..mm0.action..(friendlyname[mm0.action] or "").." - to return to options menu"},
}
		
allmenu.menutab = {
	{dysplay = "MAIN MENU:"},
	{[mm0.action] = function()
		local fileexsist =io.open(savedir.."/tank.sav", "r")
		if fileexsist then
			curmenu = allmenu.startresumegametab
			menustate = 2
			fileexsist:close()
		else
			curmenu = allmenu.startgametab
			menustate = 2
		end
	end,
	dysplay = "START GAME",
	desc = "Press: "..mm0.action..(friendlyname[mm0.action] or "").." - to start the game.\nGame controls: moving: "..mp1.up..(friendlyname[mp1.up] or "").."; "..mp1.down..(friendlyname[mp1.down] or "")..";"..mp1.left..(friendlyname[mp1.left] or "").."; "..mp1.right..(friendlyname[mp1.right] or "")..";\nshoting - "..mp1.shoot..(friendlyname[mp1.shoot] or "").." (You may change it in tank.conf)"},
	{[mm0.action] = function()
		--setmapdrawtext(mapid)
		--suboptions[3].dysplay = "MAP: "..mapdraw
		curmenu = allmenu.suboptions
		menustate = 2
	end,
	dysplay = "OPTIONS",
	desc = "Press: "..mm0.action..(friendlyname[mm0.action] or "").." - to enter options sub menu."},
	{[mm0.action]= function()
		gamestate = 6		--
	end,
	dysplay = "HIGH SCORES",
	desc = "Press: "..mm0.action..(friendlyname[mm0.action] or "").." - to see top 10 scores"},
	{[mm0.action] = function()
		setmapdrawtext(mapid)
		--selecteditormaptab[2].dysplay = "MAP: "..mapdraw
		local fileexsist =io.open(savedir.."/map.sav", "r")
		if fileexsist then
			curmenu = allmenu.resumeeditortab
			menustate = 2
			fileexsist:close()
		else
			curmenu = allmenu.starteditortab
			menustate = 2
		end
	end,
	dysplay = "MAP EDITOR",
	desc = "Press: "..mm0.action..(friendlyname[mm0.action] or "").." - to start the map editor"},
	{[mm0.action]= function()
		local imageData = love.image.newImageData("lua.png" )
		for i=0,127 do
			for j=0,127 do
				local r,g,b,a = imageData:getPixel(i,j)
				if r == 255 and g == 255 and b == 255 and a == 255 then
					imageData:setPixel(i,j,255,255,255,0)
				end
			end
		end
		imagelogo1 = love.graphics.newImage( imageData )
		imagelogo2 = love.graphics.newImage( "love1.png" )
		imagelogo3 = love.graphics.newImage( "love2.png" )
		gamestate = 11
	end,
	dysplay = "ABOUT",
	desc = "Press: "..mm0.action..(friendlyname[mm0.action] or "").." - to see the credits"},
	{[mm0.action]= function()
		love.event.push('quit')
	end,
	dysplay = "QUIT",
	desc = "Press: "..mm0.action..(friendlyname[mm0.action] or "").." - to quit the game"}
}
navtab = {
	[mm0.up] = function()
		menustate = menustate - 1
		if menustate < 2 then menustate = #curmenu end
	end,
	[mm0.down] = function()
		menustate = menustate + 1
		if menustate > #curmenu then menustate = 2 end
	end,
	[mm0.pause] = function()
		if backinmenutab[curmenu] then backinmenutab[curmenu]() end
	end,
        desc = "Menu navigation: "..mm0.up..(friendlyname[mm0.up] or "")..", "..mm0.down..(friendlyname[mm0.down] or "")..", "..mm0.pause..(friendlyname[mm0.pause] or "")
}
pausemenutab = {
	{dysplay = "PAUSE..."},
	{[mm0.action] = function()
		gamestate = 2
	end,
	dysplay = "RESUME GAME",
	desc = "Press: "..mm0.action..(friendlyname[mm0.action] or "").." - to resume the game"},
	{[mm0.action] = function()
		sawegame()
		startmenu()
	end,
	dysplay = "MAIN MENU",
	desc = "Press: "..mm0.action..(friendlyname[mm0.action] or "").." - to return to main menu"}
}
movetab = {
	[mp1.up] = function()
	   if player.slip ==0 then
		antycamp (player,0,1)
	   	isup = true
	   	isdown = false
	   	isleft = false
	   	isright = false
	   	player.grid_dir = 0
	  	pstate = 0
		rotOfsetElement(player)
		--souride:setLooping(true)
		--love.audio.play(souride)
	   end
	end,
	[mp1.down] = function()
	   if player.slip ==0 then
		antycamp (player,0,-1)
	   	isdown = true
	   	isup = false
	   	isleft = false
	  	isright = false
	  	player.grid_dir = 2
	  	pstate = 0
	  	rotOfsetElement(player)
		--souride:setLooping(true)
		--love.audio.play(souride)
	   end
	end,
	[mp1.left]= function()
	   if player.slip ==0 then
		antycamp (player,1,0)
	   	isleft = true
	   	isup = false
	   	isdown = false
	 	isright = false
	   	player.grid_dir = 3
	   	pstate = 0
	   	rotOfsetElement(player)
		--souride:setLooping(true)
		--love.audio.play(souride)
	   end
	end,
	[mp1.right]= function()
	   if player.slip ==0 then
		antycamp (player,-1,0)
	   	isright = true
	   	isup = false
	   	isdown = false
	   	isleft = false
	   	player.grid_dir = 1
	   	pstate = 0
	   	rotOfsetElement(player)
		--souride:setLooping(true)
		--love.audio.play(souride)
	   end
	end,
	[mp1.shoot] = function()
		if not player.bullet then
			if playsound then
				soushoot:stop()
				love.audio.play(soushoot)
			end
			local abullet = createBullet("pla",player.grid_x,player.grid_y,player.direction)
			player.bullet = true
			bullettab["pla"] = abullet
			rotOfsetBullet(abullet)
		end
	end,
	[mm0.pause] = function()
		if gamestate == 2 then
			gamestate = 3
			curmenu = pausemenutab 
			menustate = 2
			if playsound then
				souride:stop()
			end
		else
			startmapeditor("resume") 
		end
	end
}
backinmenutab = {
	[pausemenutab] = function()
		menustate = #pausemenutab
	end,
	[allmenu.menutab] = function()
		menustate = #allmenu.menutab
	end,
	[allmenu.dificulitytab] = function()
		curmenu = allmenu.suboptions
		menustate = 2
	end,
	[allmenu.suboptions] = function()
		curmenu = allmenu.menutab
		menustate = 2
	end,
	[allmenu.resumeeditortab] = function()
		curmenu = allmenu.menutab
		menustate = 2
	end,
	[allmenu.starteditortab] = function()
		curmenu = allmenu.menutab
		menustate = 2
	end,
	[allmenu.keysettingtab] = function()
		curmenu = allmenu.suboptions
		menustate = 2
	end,
	[allmenu.editorkeysettingtab] = function()
		curmenu = allmenu.keysettingtab
		menustate = 2
	end,
	[allmenu.menukeysettingtab] = function()
		curmenu = allmenu.keysettingtab
		menustate = 2
	end,
	[allmenu.player1keysettingtab] = function()
		curmenu = allmenu.keysettingtab
		menustate = 2
	end,
	[allmenu.startgametab]  = function()
		curmenu = allmenu.menutab
		menustate = 2
	end,
	[allmenu.dificulitybotntab] = function()
		curmenu = allmenu.dificulitytab
		menustate = 2
	end,
}
end

function loadconfigfile()
	gamestate = 0
	menustate = 2
	confprocesstab = {}
	maps = getMaps()
	
-- loading tank.conf file
	fileexsist = false
	fileexsist =io.open(directory.."/tank.conf", "r")
	if not fileexsist then 
		restoredefaultconf()
	else
		fileexsist:close()
	end

	ok, chunk = pcall( loadfile, directory.."/tank.conf") -- load the chunk safely

	if ok and not chunk then
		winstate = "file \"tank.conf\" is invalid or not exist, loading defaults..."
		table.insert(confprocesstab,#confprocesstab + 1,"file \"tank.conf\" is invalid, loading defaults...")
	else
		ok, result = pcall(chunk) 
		if not ok then 
			winstate = "The following error happened: " .. tostring(result)
			table.insert(confprocesstab,#confprocesstab + 1,"The following error happened: " .. tostring(result))
		else  
			if type(result) == "table" and result.player1_keys and type (result.player1_keys) == "table" and result.player2_keys and type (result.player2_keys) == "table" and result.menu_keys and type (result.menu_keys) == "table" and result.editor_keys and type (result.editor_keys) == "table" then
				assignkey ("up","up","all")
				assignkey ("down","down","all")
				assignkey ("left","left","all")
				assignkey ("right","right","all")
				assignkey ("shoot","shoot",mp1)
				assignkey ("action","action",mm0)
				assignkey ("pause","pause",mm0)
				assignkey ("next","next",me0)
				assignkey ("previous","previous",me0)
				assignkey ("save_quit","save_quit",me0)
				if result.maps and type(result.maps) == "table" then
					local i = #maps +1
					for j,map in pairs(result.maps) do -- loading coustom maps
						ok, chunk = pcall( loadfile, directory.."/"..map)
						if ok and not chunk then
							winstate = "file \"..map..\" is invalid or not exist..."
							table.insert(confprocesstab,#confprocesstab + 1,"map file \""..map.."\" is invalid or not exist...")
						else
							maps[i] = map
							i = i+1
						end
					end
				else
					table.insert(confprocesstab,#confprocesstab + 1,"missing seting in tank.conf (maps)")
				end
				if result.sound and tonumber(result.sound) then
					if result.sound > 0 then
						soundon() 
					else
						soundoff()
					end
						
				else
					table.insert(confprocesstab,#confprocesstab + 1,"missing seting in tank.conf (sound), or wrong value")
				end
				if result.difficulity_settings and type(result.difficulity_settings) == "table" then
					if result.difficulity_settings.small_map_size and tonumber(result.difficulity_settings.small_map_size) then
						smallmapsize = result.difficulity_settings.small_map_size
					else
						table.insert(confprocesstab,#confprocesstab + 1,"missing seting in tank.conf (small_map_size), or wrong value")
					end
					if result.difficulity_settings.max_bots_on_small_map and tonumber(result.difficulity_settings.max_bots_on_small_map) then
						slallbotn = result.difficulity_settings.max_bots_on_small_map
					else
						table.insert(confprocesstab,#confprocesstab + 1,"missing seting in tank.conf (max_bots_on_small_map), or wrong value")
					end
					if result.difficulity_settings.medium_map_size and tonumber(result.difficulity_settings.medium_map_size) then
						mediummapsize = result.difficulity_settings.medium_map_size
					else
						table.insert(confprocesstab,#confprocesstab + 1,"missing seting in tank.conf (medium_map_size), or wrong value")
					end
					if result.difficulity_settings.max_bots_on_medium_map and tonumber(result.difficulity_settings.max_bots_on_medium_map) then
						mediumbotn = result.difficulity_settings.max_bots_on_medium_map
					else
						table.insert(confprocesstab,#confprocesstab + 1,"missing seting in tank.conf (max_bots_on_medium_map), or wrong value")
					end
					if result.difficulity_settings.large_map_size and tonumber(result.difficulity_settings.large_map_size) then
						largemapsize = result.difficulity_settings.large_map_size
					else
						table.insert(confprocesstab,#confprocesstab + 1,"missing seting in tank.conf (large_map_size), or wrong value")
					end
					if result.difficulity_settings.max_bots_on_large_map and tonumber(result.difficulity_settings.max_bots_on_large_map) then
						largebotn = result.difficulity_settings.max_bots_on_large_map
					else
						table.insert(confprocesstab,#confprocesstab + 1,"missing seting in tank.conf (max_bots_on_large_map), or wrong value")
					end
					if result.difficulity_settings.max_bots_on_very_large_map and tonumber(result.difficulity_settings.max_bots_on_very_large_map) then
						verylalgebotn = result.difficulity_settings.max_bots_on_very_large_map
					else
						table.insert(confprocesstab,#confprocesstab + 1,"missing seting in tank.conf (max_bots_on_very_large_map), or wrong value")
					end
					if result.difficulity_settings.starting_amount_of_bots and tonumber(result.difficulity_settings.starting_amount_of_bots) then
						storebotleft = result.difficulity_settings.starting_amount_of_bots
					else
						table.insert(confprocesstab,#confprocesstab + 1,"missing seting in tank.conf (starting_amount_of_bots), or wrong value")
					end
					if result.difficulity_settings.new_bots_per_map and tonumber(result.difficulity_settings.new_bots_per_map) then
						botincreeseamount = result.difficulity_settings.new_bots_per_map
					else
						table.insert(confprocesstab,#confprocesstab + 1,"missing seting in tank.conf (new_bots_per_map), or wrong value")
					end
					if result.difficulity_settings.player_base_hp and tonumber(result.difficulity_settings.player_base_hp) then
						playerbasehp = result.difficulity_settings.player_base_hp
					else
						table.insert(confprocesstab,#confprocesstab + 1,"missing seting in tank.conf (player_base_hp), or wrong value")
					end
					if result.difficulity_settings.bot_base_hp and tonumber(result.difficulity_settings.bot_base_hp) then
						botbasehp = result.difficulity_settings.bot_base_hp
					else
						table.insert(confprocesstab,#confprocesstab + 1,"missing seting in tank.conf (bot_base_hp), or wrong value")
					end
					if result.difficulity_settings.bullet_anihilation and tonumber(result.difficulity_settings.bullet_anihilation) then
						if result.difficulity_settings.bullet_anihilation > 0 then
							bulletandisplay = "ON"
						else
							bulletandisplay = "OFF"
						end
					else
						table.insert(confprocesstab,#confprocesstab + 1,"missing seting in tank.conf (bullet_anihilation), or wrong value")
					end
					if result.difficulity_settings.fire_detector and tonumber(result.difficulity_settings.fire_detector) then
						if result.difficulity_settings.fire_detector > 0 then
							firedetectordisplay = "ON"
						else
							firedetectordisplay = "OFF"
						end
					else
						table.insert(confprocesstab,#confprocesstab + 1,"missing seting in tank.conf (fire_detector), or wrong value")
					end

				else
					table.insert(confprocesstab,#confprocesstab + 1,"difficulity settings are missing from file \"tank.conf\"")
				end
			else
				winstate = "unexpected table format in file \"tank.conf\""
				table.insert(confprocesstab,#confprocesstab + 1,"unexpected table format in file \"tank.conf\"")
			end
		end
	end 
	reloadmenu()
	if #confprocesstab == 0 then
		startmenu()
		local sctualconfigfunc = loadfile(directory.."/tank.conf")
		actualconfig = sctualconfigfunc()
		local maplistbackup = actualconfig.maps
		local text = "return "..table.tostring(maplistbackup)
		love.filesystem.write("maplist.bck",text)

		local file = io.open("tank.conf", "r")
		local text = file:read("*a")

		love.filesystem.write("tank.conf",text)
		file:close()
	else
		table.insert(confprocesstab,1,"Processing \"tank.conf\" file failed:")
		table.insert(confprocesstab,#confprocesstab + 1,"Game will use default settings (if necesery)")
		table.insert(confprocesstab,#confprocesstab + 1,"Press any key to continue")
	end
end

math.randomseed(os.time())
math.random()
math.random()

function startmapeditor(typ,amap,w,h) -- new rssume map previous
	allmenu = {}
	editorcursor.name = ""
	editorcursor.errorname = ""
	if typ == "new" then
		
		editorcursor.id = -1

		map = {}
		for i=1,h do
			map[i]={}
			for j=1,w do 
				if i == 1 or j == 1 or i == h or j == w then
					map[i][j] = 2
				else
					map[i][j] = 0
				end
			end
		end
		map[2][2] = -2
		map[2][3] = -1
		map[2][5] = -3
		editorcursor.plaspawn[1]= {x=3,y=2}
		editorcursor.botspawn[1]= {x=2,y=2}
		editorcursor.basespawn[1]= {x=5,y=2}
	elseif typ == "resume" then
		local func = loadfile(savedir.."/map.sav")
		local tab = func()
		map = tab.map
		editorcursor = tab.editorcursor
	else
		local mapname = maps[mapid]
		if tonumber(mapdraw) then
			ok, chunk = pcall( love.filesystem.load, mapname)
		else
			ok, chunk = pcall( loadfile, directory.."/"..mapname)
		end
		if not ok and type(chunk)=="string" then --(edit-30-03-12)
			winstate = "map file \""..mapname.."\" is invalid or not exist..."
			if atype == "next" then
				return 
			end
		else
			map = chunk()
			editorcursor.id = 0
			local jn = 0
			for i=1,#map do
				for j=1,#map[1] do 
					if map[i][j] == -1 then
						editorcursor.plaspawn[1]= {x=j,y=i}	
					elseif map[i][j] == -2 then
						jn = jn+1
						editorcursor.botspawn[jn] = {x=j,y=i}
					elseif map[i][j] == -3 then
						editorcursor.basespawn[1]= {x=j,y=i}
					end
				end
			end

		end
	end
	imagegrid = love.graphics.newSpriteBatch(imageframe,#map*#map[1])
	for i=1,#map do
		for j=1,#map[1] do 
			imagegrid:add(j*32,i*32)
		end
	end
	boundcamera()
	editorcursor.x = 2
	editorcursor.y = 2
	gamestate = 8
end

function sawegame()
	local savetab = {map=map,dynamicmap=dynamicmap,spawnmap=spawnmap,bottab=bottab,bullettab=bullettab,player=player,mapid=mapid,botn=botn,botleft=botleft,botincreese=botincreese,spottab=spottab,tempspottab=tempspottab,gamemodestate=gamemodestate,playerbase=playerbase,basetab=basetab}
	local savetext = "return "..table.tostring(savetab)
	savetext = string.gsub (savetext,"={","={   \n") 
	savetext = string.gsub (savetext,"},","},   \n")
	local newfile = io.open(savedir.."/tank.sav", "w")
	newfile:write(savetext)
	newfile:close()
end

function loadgame()
	local func = loadfile(savedir.."/tank.sav")
	local tab = func()
	map= tab.map
	dynamicmap= tab.dynamicmap
	spawnmap= tab.spawnmap
	bottab= tab.bottab	
	bullettab= tab.bullettab
	player= tab.player
	mapid= tab.mapid
	botn= tab.botn
	botleft= tab.botleft
	spottab = tab.spottab
	tempspottab = tab.tempspottab
	botincreese= tab.botincreese
	gamemodestate = tab.gamemodestate
	playerbase = tab.playerbase
	basetab = tab.basetab
	setmapdrawtext(mapid) --(edit-30-03-12)
	--suboptions[3].dysplay = "MAP: "..mapdraw --(edit-30-03-12) --warning
	imagebackground = love.graphics.newSpriteBatch(imageatlas,#map*#map[1])
	for i=1,#map do
		for j=1,#map[1] do 
			if map[i][j] == 2 then
				imagebackground:addq(imagewall,j*32,i*32)
			else 
				imagebackground:addq(imagegras,j*32,i*32)
			end
		end
	end
	boundcamera()
	for i,bot in pairs(bottab) do
		bot.decisionlag = 0
		bot.dete = 0
		--bot.soushoot = love.audio.newSource( "shoot.mp3")
	end 
	--[[for i,bullet in pairs(bullettab) do
		bullet.souhit = love.audio.newSource( "hit.mp3")
	end]]
	gamestate = 2
end

function love.quit()
	if gamestate == 2 or gamestate == 3 then
		sawegame()
	elseif gamestate == 8 then
		saveeditorstate ()
	end
end

function saveeditorstate ()
	local tab = {map=map,editorcursor=editorcursor}
	local savetext = "return "..table.tostring(tab)
	savetext = string.gsub (savetext,"={","={   \n") 
	savetext = string.gsub (savetext,"},","},   \n")
	local newfile = io.open(savedir.."/map.sav", "w")
	newfile:write(savetext)
	newfile:close()
end

function startgame(atype,botsleft,testlvl,testhp) -- starting the game 
	--souride:setLooping(true)
	if atype ~= "test" then
		map = {}
	end
	allmenu = {}
	dynamicmap = {}
	bottab = {}
	bullettab = {}
	local startx = 0
	local starty = 0
	spawnmap = {}
	spottab = {}
	tempspottab = {}
	basetab = {}
	local jn = 0
	if atype ~= "next" then
		botleft = storebotleft
	end
	local mapname = maps[mapid]
	if tonumber(mapdraw) then
		ok, chunk = pcall( love.filesystem.load, mapname)
	else
		ok, chunk = pcall( loadfile, directory.."/"..mapname)
	end
	if not ok and type(chunk)=="string" then --(edit-30-03-12)
		winstate = "map file \""..mapname.."\" is invalid or not exist..."
		if atype == "next" then
			return nextmap()
		end
	else
		if atype ~= "test" then
			map = chunk()
		end
		for i=1,#map do
			dynamicmap[i] = {}
			for j=1,#map[1] do 
				dynamicmap[i][j] = 0
				if map[i][j] == -1 then
					startx = j
					starty = i
					testtest = -1
				elseif map[i][j] == -2 then
					jn = jn+1
					spawnmap[jn] = {x=j,y=i}
					testtest = -2
				elseif map[i][j] == 9 or map[i][j] == 10 then
					createSpoot("map", j, i)
				elseif map[i][j] == -3 then
					if (gamemodestate == 2 or gamemodestate == 4) then
						playerbase = createPlayerBase(playerbasehp,j,i)
					else
						map[i][j] = 0
					end
				end
			end
		end
		if atype == "next" then
			player.grid_x = 32*startx
			player.grid_y = 32*starty
			player.act_x = 32*startx
			player.act_y = 32*starty
		elseif atype == "new" then
			player = createPlayer(startx,starty,0,1)
		elseif atype == "test" then
			player = createPlayer(startx,starty,testlvl,testhp)
		end 
		local j = 1
		local mapsize = #map*#map[1]
		
		if mapsize <= smallmapsize then
			botn = slallbotn
		elseif mapsize <= mediummapsize then
			botn = mediumbotn
		elseif mapsize <= largemapsize then
			botn = largebotn
		else
			botn = verylalgebotn
		end
		if (gamemodestate == 3 or gamemodestate == 4) then
			local basen = math.min(math.floor((player.maxhp+1)/2),4)
			for i=1,basen do
				local base = createbotbase(spawnmap[math.random(1,#spawnmap)],botbasehp+player.level,i)
				basetab[i] = base
			end
			botleft = botn
		else
			botn = math.min(botn,botleft)
		end
		
    		for i=1,botn do
			local maxhp = math.random(1,math.max(player.level-1,1))
			local bot = createBot(spawnmap[j],maxhp,maxhp,i)
    			if j == jn then j = 1 else j = j+1 end
      			bottab[i] = bot
   		end
    	--botleft = botn
	imagebackground = love.graphics.newSpriteBatch(imageatlas,#map*#map[1])   --#map*#map[1]
	for i=1,#map do --#map
		for j=1,#map[1] do --#map[1]
			if map[i][j] == 2 then
				imagebackground:addq(imagewall,j*32,i*32)
			else 
				imagebackground:addq(imagegras,j*32,i*32)
			end
		end
	end
 	boundcamera()
	if atype == "test" then
		gamestate = 10
	else
    	gamestate = 2
	end
	end
end

function boundcamera()
	local x = #map[1]*32-width+32*5
    	if x < 0 then x = 0 end
    	local y = #map*32-height+32*2
    	if y < 0 then y = 0 end
    	camera:setBounds(0, 0, x, y)
end

local text = ""
local text2 = ""
local pstate = 0
local pdm = 0

function startmenu() -- loading the menu
	reloadmenu()
	gamestate = 1
	curmenu = allmenu.menutab
	menustate = 2
	bottab = {}
	bullettab = {}
	--botn = dificulitytab[2].botn  -- warning
	--botleft = dificulitytab[3].botleft --warning
	--botincreeseamount = dificulitytab[4].botincreeseamount
end

function createPlayerBase(basehp,posx,posy)
	for i=-1,1 do
		for j=-1,1 do 
			local xx = posx + i
			local yy = posy + j
			if not (xx == 1 or xx == #map[1] or yy == 1 or yy == #map or map[yy][xx] == -1 or map[yy][xx] == -2 or map[yy][xx] == -3) then
				map[yy][xx] = 3
			end
		end
	end
	return {
		hp = basehp,
		x = posx,
		y = posy
	}
end

function createPlayer(startx,starty,alvl,ahp) -- new player element
	return {
		atype = "player",
        	grid_x = 32*startx,
       		grid_y = 32*starty,
        	act_x = 32*startx,
        	act_y = 32*starty,
		draw_x = 0,
        	draw_y = 0,
		score = 0,
		ismoving = {x=false,y=false,overal=false,overalstate=true},
		basespeed = 2,
        	speed = 2,
		grid_dir = 0,
		direction = 0,
		bullet = false,
		hp = ahp,
		maxhp = ahp,
		kills = 0,
		level = alvl,
		nextlevel = 5,
		slip = 0,
		slipstate = false,
		name = ""
        }
end

function createbotbase(spawn,ahp,i)
	return {
		atype = "base",
		i = i,
		grid_x = spawn.x*32,
       	grid_y = spawn.y*32,
       	act_x = spawn.x*32,
       	act_y = spawn.y*32,
		ismoving = {x=false,y=false,overal=false,overalstate=true},
		draw_x = 0,
       	draw_y = 0,
		direction = 0,
		grid_dir = 0,
		randominterval = 5.5,
		decisionlag = 0,
		hp = ahp,
		basespeed = 1,
        	speed = 1,
		randomdt = 0,
		slip = 0,
	}
end

function createBot (spawn,ahp,amaxhp,i) -- new enemy element
	return {
		atype = "bot",
		i = i,
	    grid_x = spawn.x*32,
       	grid_y = spawn.y*32,
       	act_x = spawn.x*32,
       	act_y = spawn.y*32,
		ismoving = {x=false,y=false,overal=false,overalstate=true},
		draw_x = 0,
       	draw_y = 0,
		basespeed = 2,
       	speed = 2,
	   	direction = 0,
		grid_dir = 0,
		bullet = false,
		dete = 0,
		randomdt = 0,
		randominterval = 5.5,
		decisionlag = 0,
		hp = ahp,
		maxhp = amaxhp,
		slip = 0,
		visible = true,
		respawn = 10
        }
end

function createBullet(id,x,y,dire) --new projectale element
	return {
		atype = "bullet",
		botid = id,
		grid_x = x,
		grid_y = y,
		act_x = x,
		act_y = y,
		draw_x = 0,
		acurat = false,
		ismoving = {x=false,y=false,overal=false,overalstate=true},
		draw_y = 0,
		speed = 4,
		dir = dire,
		range = 0
	       }
end

function createSpoot(ori, px, py) -- ori: "map" "fir" "law" "bot" "timeout"
	local id = (py-1)*#map[1]+px
	local block = map[py][px]
	local count = false
	local typ = "permfir"
	if ori == "map" then
		count = false
		if block == 10 then
			typ = "permfir"
		else
			typ = "permlaw"
		end
	elseif ori == "timeout" then
		block = 0
		count = 6
		typ = "fir"
	else
		if spottab[id] and spottab[id].typ == "permfir" then
			block = 9
			count = false
			typ = "permlaw"
		elseif spottab[id] and spottab[id].typ == "permlaw" then
			block = 9
			count = false
			typ = "permlaw"
		elseif spottab[id] and spottab[id].typ == "fir" then
			block = 9
			count = 6
			typ = "law"
		elseif spottab[id] and spottab[id].typ == "law" then
			block = 9
			count = 6
			typ = "law"
		elseif block == 8 then
			block = 5
			count = 6
			typ = "fir"
		elseif block == 7 then
			block = 10
			count = false
			typ = "permfir"
		elseif block == 6 then
			block = 1
			count = 2
			typ = "fir"
		elseif block == 5 then
			block = 4
			count = 1
			typ = "fir"
		elseif block == 4 then
			block = 3
			count = 1
			typ = "fir"
		elseif block == 3 then
			block = 11
			count = 1
			typ = "fir"
		elseif block == 2 then
			return
		elseif block == 1 then
			block = 7
			count = 2
			typ = "fir"
		elseif block == 0 or block == -1 or block == -2 then
			block = 0
			count = 6
			typ = "fir"
		elseif block == 11 then
			block = 11
			count = 1
			typ = "fir"
		elseif block == 12 then
			block = 11
			count = 1
			typ = "fir"
		elseif block == -3 then
			return
		end

	end
	if typ == "fir" or typ == "permfir" then
		map[py][px] = 10
	else	
		map[py][px] = 9
	end
	local spot = {
		ori = ori,
		typ = typ,
		block = block,
		x = px,
		y = py,
		count = count,

	}
	spottab[id]=spot
	if typ == "fir" or typ == "law" then
		tempspottab[id] = spot
	end
end

local antycampmap = {}
function savedynamic(bot,x,y) -- colisions of tanks
	dynamicmap[(bot.grid_y / 32)+y][(bot.grid_x / 32)+x] = 0
	if bot.atype == "player" then
		dynamicmap[(bot.grid_y / 32)][(bot.grid_x / 32)] = 3
		antycamp (bot,x,y)
	else
		dynamicmap[(bot.grid_y / 32)][(bot.grid_x / 32)] = 1
	end
end
function antycamp (bot,x,y) -- aditional colisions for layer
		for i,tab in pairs(antycampmap) do
			if tab and type(tab) == "table" then
				if dynamicmap[tab.y] and dynamicmap[tab.y][tab.x] == 2 then
						dynamicmap[tab.y][tab.x] = 0
				end
			end
		end
		if dynamicmap[(bot.grid_y / 32)-y] and dynamicmap[(bot.grid_y / 32)-y][(bot.grid_x / 32)-x] == 0 then
			dynamicmap[(bot.grid_y / 32)-y][(bot.grid_x / 32)-x] = 2		
			if dynamicmap[(bot.grid_y / 32)-y*2] and dynamicmap[(bot.grid_y / 32)-y*2][(bot.grid_x / 32)-x*2] == 0 then
				dynamicmap[(bot.grid_y / 32)-y*2][(bot.grid_x / 32)-x*2] = 2
				if dynamicmap[(bot.grid_y / 32)-y*3] and dynamicmap[(bot.grid_y / 32)-y*3][(bot.grid_x / 32)-x*3] == 0 then
					dynamicmap[(bot.grid_y / 32)-y*3][(bot.grid_x / 32)-x*3] = 2
				end
			end
		end
		antycampmap = {{x=(bot.grid_x / 32)-x,y=(bot.grid_y / 32)-y},
			       {x=(bot.grid_x / 32)-x*2,y=(bot.grid_y / 32)-y*2},
			       {x=(bot.grid_x / 32)-x*3,y=(bot.grid_y / 32)-y*3}}
end
--local isplayermoving = {x=false,y=false,overal=false,overalstate=true}
function setisplayermowing(elem,state,dir)
    if elem.ismoving then
	elem.ismoving[dir] = state
	if not elem.ismoving.x  and not elem.ismoving.y then
		elem.ismoving.overal=false
	else
		elem.ismoving.overal=true
	end
	if elem.ismoving.overal and elem.ismoving.overalstate then
		elem.ismoving.overalstate = false
		--souride:setLooping(true)
		if playsound and elem.atype == "player" then
			love.audio.play(souride)
		end
	elseif not elem.ismoving.overal and not elem.ismoving.overalstate then
		elem.ismoving.overalstate = true
		--souride:setLooping(false)
		if playsound and elem.atype == "player" then
			souride:stop()
		end
	end
   end
end
function moveElement(elem,dt,dir) --smoudh moving
   if not dir then
	moveElement(elem,dt,"x")
	moveElement(elem,dt,"y")
   else
  	 if elem["act_"..dir] - elem["grid_"..dir] < 2*(32*elem.speed*dt) and elem["grid_"..dir] - elem["act_"..dir] < 2*(32*elem.speed*dt) then
 	     elem["act_"..dir] = elem["grid_"..dir]
	     setisplayermowing(elem,false,dir)
 	  elseif not (elem["act_"..dir] - elem["grid_"..dir] < 2*(32*elem.speed*dt)) then
 	     elem["act_"..dir] = elem["act_"..dir] - 32*elem.speed*dt
	     setisplayermowing(elem,true,dir)
 	  elseif not (elem["grid_"..dir] - elem["act_"..dir] < 2*(32*elem.speed*dt)) then
 	     elem["act_"..dir] = elem["act_"..dir] + 32*elem.speed*dt
	     setisplayermowing(elem,true,dir)
 	  end
   end
end

function moveGrid(elem,dir1,dir2,dir3,dir4) -- actual position change
	if elem.act_y == elem.grid_y and elem.act_x == elem.grid_x  then
        	elem["grid_"..dir1] = elem["grid_"..dir1] + dir2
		if elem.atype == "bullet" then
			elem.range = elem.range + 1
		else
			elem.direction = elem.grid_dir
			savedynamic(elem,dir3,dir4)
			rotOfsetElement(elem)
			if elem.atype == "player" then
				--updateTilesetBatch()
			end
		end
	end
end

function rotOfsetElement(elem) -- rotations for tanks
	elem.direction = elem.grid_dir
	if elem.direction == 2 then 
		elem.draw_x = 32
		elem.draw_y = 32
	elseif elem.direction == 3 then
		elem.draw_y = 32
		elem.draw_x = 0
	elseif elem.direction == 1 then
		elem.draw_x = 32
		elem.draw_y = 0
	elseif elem.direction == 0 then
		elem.draw_x = 0
		elem.draw_y = 0
	end
end

function rotOfsetBullet(bullet) -- rotations for projectals
	if bullet.dir  == 0 then
		bullet.draw_x = 16 
		bullet.draw_y = 0 
	elseif bullet.dir == 1 then
		bullet.draw_x = 32 
		bullet.draw_y = 16 
	elseif bullet.dir == 2 then
		bullet.draw_x = 16 
		bullet.draw_y = 32 
	else
		bullet.draw_y = 16 
		bullet.draw_x = 0 
	end
end

local updatefunc = {} 
local cursorinterval = 0.8
local cursordt = 0
local cursordysplay = ""
local cursorstate = true
updatefunc[7] = function(dt)
	cursordt = cursordt + dt
	if cursordt > cursorinterval then
		cursordt = 0
		cursorstate = not cursorstate
		if cursorstate then
			cursordysplay = ""
		else
			cursordysplay = "|"
		end
	end	
end
updatefunc[9] = function(dt)
	updatefunc[7](dt)
end
--[[function stopridesound()
	souride:setLooping(false)
	souride:stop() 
end]]

local stepmousx = 0
local spepmousy = 0
local mosemodx = false
local mosemody = false
function love.mousepressed( x, y, button )
	editorkeys(button)
end

function editorkeys(button)
	if button == "wu" or button == me0.next then
		editorcursor.id = editorcursor.id + 1
		if editorcursor.id > 11 then editorcursor.id = -3 end
	elseif button == "wd" or button == me0.previous then
		editorcursor.id = editorcursor.id - 1
		if editorcursor.id < -3 then editorcursor.id = 11 end
	elseif button == me0.save_quit then
		saveeditorstate ()
		love.graphics.setColorMode("modulate")
		startmenu()
		curmenu = allmenu.savequiteditortab
		menustate = 2
	end
end

updatefunc[8] = function (dt) --editor
	editorcursor.blink = editorcursor.blink - dt
	if editorcursor.blink < 0 then
		editorcursor.blink = 0.4
		editorcursor.blinkstate = not editorcursor.blinkstate
	end
	if love.keyboard.isDown(me0.up) then
		editorcursor.mapposy = editorcursor.mapposy - 200*dt
	elseif love.keyboard.isDown(me0.down) then
		editorcursor.mapposy = editorcursor.mapposy + 200*dt
	elseif love.keyboard.isDown(me0.left) then
		editorcursor.mapposx = editorcursor.mapposx - 200*dt
	elseif love.keyboard.isDown(me0.right) then
		editorcursor.mapposx = editorcursor.mapposx + 200*dt
	end
	if editorcursor.mapposy < height / 2 then editorcursor.mapposy = height/2 end
	if editorcursor.mapposy > camera._bounds.y2 + height/2 then editorcursor.mapposy = camera._bounds.y2 + height/2 end
	if editorcursor.mapposx < width / 2 then editorcursor.mapposx = width/2 end
	if editorcursor.mapposx > camera._bounds.x2 + width / 2 then editorcursor.mapposx = camera._bounds.x2 + width / 2 end

	local pcx,pcy = editorcursor.x,editorcursor.y
	local mx,my = love.mouse.getPosition( )
	mx = mx + editorcursor.mapposx - width / 2
	my = my + editorcursor.mapposy - height / 2
	editorcursor.y = math.modf(my/32)
	if not map[editorcursor.y] or editorcursor.y == 1 or editorcursor.y == #map then 
		editorcursor.y = pcy
	else
		editorcursor.x = math.modf(mx/32) 
		if not map[editorcursor.y][editorcursor.x] or editorcursor.x == 1 or editorcursor.x == #map[1] then 
			editorcursor.x = pcx
		end
	end
	if love.mouse.isDown( "l" ) then
		if editorcursor.id == -1 and (map[editorcursor.y][editorcursor.x] ~= -2 and map[editorcursor.y][editorcursor.x] ~= -3) then
			map[editorcursor.plaspawn[1].y][editorcursor.plaspawn[1].x] = 0
			editorcursor.plaspawn[1].y = editorcursor.y
			editorcursor.plaspawn[1].x = editorcursor.x
		elseif editorcursor.id == -2 and (map[editorcursor.y][editorcursor.x] ~= -1 and map[editorcursor.y][editorcursor.x] ~= -3) then
			for i,spawn in pairs(editorcursor.botspawn) do
				if spawn.x == editorcursor.x and spawn.y == editorcursor.y then
					return
				end
				
			end
			editorcursor.botspawn[#editorcursor.botspawn + 1] = {y = editorcursor.y, x = editorcursor.x}
		elseif editorcursor.id == -3 and (map[editorcursor.y][editorcursor.x] ~= -2 and map[editorcursor.y][editorcursor.x] ~= -1) then
			map[editorcursor.basespawn[1].y][editorcursor.basespawn[1].x] = 0
			editorcursor.basespawn[1].y = editorcursor.y
			editorcursor.basespawn[1].x = editorcursor.x 
		elseif editorcursor.id == 0 then
			return editorremovetile()
		end
		if map[editorcursor.y][editorcursor.x] > -1 then
			map[editorcursor.y][editorcursor.x] = editorcursor.id
		end
	elseif love.mouse.isDown( "r" ) then
		editorremovetile()
	end
	camera:setPosition(editorcursor.mapposx - width / 2, editorcursor.mapposy - height / 2)

end

function editorremovetile()
	if map[editorcursor.y][editorcursor.x] ~= -1 and map[editorcursor.y][editorcursor.x] ~= -3 then
		if  map[editorcursor.y][editorcursor.x] == -2 and #editorcursor.botspawn > 1 then
			for i,spawn in pairs(editorcursor.botspawn) do
				if spawn.x == editorcursor.x and spawn.y == editorcursor.y then
					table.remove(editorcursor.botspawn,i)
					break
				end
				
			end
			map[editorcursor.y][editorcursor.x] = 0
		elseif map[editorcursor.y][editorcursor.x] ~= -2 then
			map[editorcursor.y][editorcursor.x] = 0
		end
	end
end

local animinrerval = 0
local animduration = 1.2
local animcorect = 0
local firspreedtimer = 12

local idnotspreddingfire = {[1]=true,[3]=true,[4]=true,[5]=true,[6]=true,[11]=true,[12]=true}

updatefunc[10] = function (dt)
	updatefunc[2](dt)
end

updatefunc[2] = function (dt) -- game tempspottab
	for i,spot in pairs (tempspottab) do
		if spot.count then
			spot.count = spot.count - dt
			if spot.count < 0 then
				if spot.typ == "law" then
					createSpoot("timeout", spot.x, spot.y)
				else
					map[spot.y][spot.x] = spot.block
					local rand = math.random(0,4)
					if rand <= 1 and not idnotspreddingfire[spot.block] then
						rand = math.random(0,3)
						if rand == 0 then createSpoot("fir", spot.x ,spot.y - 1)
						elseif rand == 2 then createSpoot("fir", spot.x, spot.y + 1)
						elseif rand == 3 then createSpoot("fir", spot.x - 1, spot.y)
						elseif rand == 1 then createSpoot("fir", spot.x + 1, spot.y) end
					end
					spottab[i] = nil
					tempspottab[i] = nil
				end
			end 
		end
	end	
	firspreedtimer = firspreedtimer - dt
	if firspreedtimer < 0 then
		firspreedtimer = 12
		for i,spot in pairs (spottab) do
			if spot.typ == "law" or spot.typ == "permlaw" then
				local rand = math.random(1,2)
				if rand == 1 then
					rand = math.random(0,3)
					if rand == 0 then createSpoot("law", spot.x ,spot.y - 1)
					elseif rand == 2 then createSpoot("law", spot.x, spot.y + 1)
					elseif rand == 3 then createSpoot("law", spot.x - 1, spot.y)
					elseif rand == 1 then createSpoot("law", spot.x + 1, spot.y) end
				end
			elseif not tempspottab[i] then
				local rand = math.random(0,9)
				if rand == 0 then
					rand = math.random(0,3)
					if rand == 0 then createSpoot("fir", spot.x ,spot.y - 1)
					elseif rand == 2 then createSpoot("fir", spot.x, spot.y + 1)
					elseif rand == 3 then createSpoot("fir", spot.x - 1, spot.y)
					elseif rand == 1 then createSpoot("fir", spot.x + 1, spot.y) end
				end
			end
		end
	end
	animinrerval = animinrerval + dt
	if animinrerval > animduration then
		animinrerval = 0
		scalexbus = 0 - scalexbus
		if scalexbus > 0 then
			animcorect = 0
		else
			animcorect = 32
		end
	end 
	camera:setPosition(player.act_x - width / 2, player.act_y - height / 2)
	moveElement(player,dt)
    	text = dt
	if player.ismoving then
    	  if (not player.ismoving.overal) and isup and testBotMap(player,0, -1) then moveGrid(player,"y",-32,0,1) end
    	  if (not player.ismoving.overal) and isdown and testBotMap(player,0, 1) then moveGrid(player,"y",32,0,-1) end
    	  if (not player.ismoving.overal) and isleft and testBotMap(player,-1, 0) then moveGrid(player,"x",-32,1,0) end
    	  if (not player.ismoving.overal) and isright and testBotMap(player,1, 0) then moveGrid(player,"x",32,-1,0)  end
	end
    	for i,bot in pairs( bottab )do	
		bothandler(bot,dt,i)
	end
	for i,base in pairs(basetab) do
		bothandler(base,dt,i)
	end
	for i,bullet in pairs(bullettab)  do
		
		if (not bullet.ismoving.overal) and bullet.dir == 0 and testbulMap(bullet,0,-1,0,-2,-1,-1,1,-1) then moveGrid(bullet,"y",-32,0,0)
		elseif (not bullet.ismoving.overal) and bullet.dir == 2 and testbulMap(bullet,0,1,0,2,1,1,-1,1) then moveGrid(bullet,"y",32,0,0)
		elseif (not bullet.ismoving.overal) and bullet.dir == 3 and testbulMap(bullet,-1,0,-2,0,-1,1,-1,-1) then moveGrid(bullet,"x",-32,0,0)
		elseif (not bullet.ismoving.overal) and bullet.dir == 1 and testbulMap(bullet,1,0,2,-2,1,-1,1,1) then moveGrid(bullet,"x",32,0,0)
		elseif (not bullet.ismoving.overal) then
			if i ~= "pla" and bottab[i] then
				bottab[i].bullet = false
			elseif i == "pla" then
				player.bullet = false
			end
			if playsound and isElementOnScreen(bullet.act_x,bullet.act_y) then
					local distance = (((bullet.act_x-player.act_x)^2)+((bullet.act_y-player.act_y)^2))^0.5
					if distance < 15*32 then
						bulletsouhit:stop()
						love.audio.play(bulletsouhit)
						--bulletsouhit = nil
					end
				end
			--love.audio.play(bullet.souhit)
			
			bullettab[i] = nil
	    	end
		if bullet.range >= 12 then 
			if i ~= "pla" and bottab[i] then
				bottab[i].bullet = false
			elseif i == "pla" then
				player.bullet = false
			end
			bullettab[i] = nil
		end
		moveElement(bullet,dt)
		--if bullet.grid_x == player.grid_x and bullet.grid_y == player.grid_y and i ~= "pla" then
		if i ~= "pla" and bullet.acurat and didbullethit(bullet,player) then
			bullettab[i] = nil
			if i ~= "pla" and bottab[i] then
				bottab[i].bullet = false
			end
			damageplayer ()
		end
		if bulletandisplay == "ON" and i ~= "pla" and bullettab.pla and didbullethit(bullet,bullettab.pla) then
			bullettab[i] = nil
			bottab[i].bullet = false
			bullettab.pla = nil
			player.bullet = false
		end
	 end
end 

function bothandler(bot,dt,i)
	bot.randomdt = bot.randomdt + dt
    		if bot.randomdt >= bot.randominterval and bot.slip == 0 then
			bot.randomdt = 0
			bot.grid_dir = math.random(0,3)
			rotOfsetElement(bot)
    		end
		if (not bot.ismoving.overal) and bot.grid_dir == 0 and testBotMap(bot,0, -1) then moveGrid(bot,"y",-32,0,1)
		elseif (not bot.ismoving.overal) and bot.grid_dir == 2 and testBotMap(bot,0, 1) then moveGrid(bot,"y",32,0,-1)
		elseif (not bot.ismoving.overal) and bot.grid_dir == 3 and testBotMap(bot,-1, 0) then moveGrid(bot,"x",-32,1,0)
		elseif (not bot.ismoving.overal) and bot.grid_dir == 1 and testBotMap(bot,1, 0) then moveGrid(bot,"x",32,-1,0)
		elseif (not bot.ismoving.overal) and bot.act_y == bot.grid_y and bot.act_x == bot.grid_x then
			local atime = love.timer.getTime()
			if (atime - bot.decisionlag) > 0.5 then
				bot.decisionlag = atime
				bot.grid_dir = math.random(0,3)
				rotOfsetElement(bot)
			else
				bot.grid_dir = 4
			end
		end
		moveElement(bot,dt)
	    
        	if not bot.bullet and bot.atype == "bot" then
			local atime = love.timer.getTime()
			local distance = (((bot.act_x-player.act_x)^2)+((bot.act_y-player.act_y)^2))^0.5
			if (atime - bot.dete) > 1 and distance < 15*32 then
				bot.dete = atime
				if playsound and isElementOnScreen(bot.act_x,bot.act_y) then
					--distance = (((bot.act_x-player.act_x)^2)+((bot.act_y-player.act_y)^2))^0.5
					if distance < 12*32 then
						botsoushoot:stop()
						love.audio.play(botsoushoot)
					end
				end
				local abullet = createBullet(i,bot.grid_x,bot.grid_y,bot.direction)
				bot.bullet = true
				bullettab[i] = abullet
				rotOfsetBullet(abullet)
			end
		end
		--if bullettab["pla"] and bullettab["pla"].grid_x == bot.grid_x and bullettab["pla"].grid_y == bot.grid_y then
		if bullettab["pla"] and bullettab["pla"].acurat and didbullethit(bullettab["pla"],bot) then
			--if bot.atype == "bot" then
				bullettab["pla"] = nil
				player.bullet = false
				damagebot(bot)
			--end
		end
end

function playerkillplus()
					player.kills = player.kills + 1
					if player.kills >= player.nextlevel then 
						player.level = player.level + 1
						player.maxhp = player.maxhp + 1
						if player.level > 99 then player.level = 99 end
						if player.maxhp > 99 then player.maxhp = 99 end
						player.hp = player.maxhp
						player.nextlevel = math.floor(player.nextlevel*1.5 + 3)
						for i,bot in pairs(bottab) do
							bot.maxhp = math.random(1,math.max(player.level-1,1))
							bot.hp = bot.maxhp
						end
					end	
end

function winstatehandler()
	if gamestate == 2 then
			   			winstate = "VICTORY"
						if playsound then
							souride:stop()
						end
						player.slip = 0
						restorecontroll (player)
						screentimer = 5
			  			gamestate = 4
	else 
						startmapeditor("resume")
	end
end

function damagebot(bot,am)
			bot.hp = bot.hp - (am or 1)
			if not am and gamestate == 2 then player.score = player.score + 3 end
			if playsound then
				souhit:stop()
				love.audio.play(souhit)
			end
			if bot.hp <= 0 then
				if (gamemodestate == 1 or gamemodestate == 2) then
					botleft = botleft -1
				elseif bot.atype == "base" then
					playerkillplus()
					basetab[bot.i] = nil
					local basen = 0
					for i,base in pairs(basetab) do
						basen = basen +1
					end
					if basen == 0 then
						winstatehandler()
					end
				end
				createSpoot("bot", bot.grid_x/32, bot.grid_y/32)
				if not am and gamestate == 2 then player.score = player.score + 2 end
				if playsound then
					souboom:stop()
					love.audio.play(souboom)
				end
				dynamicmap[(bot.grid_y / 32)][(bot.grid_x / 32)] = 0
				if not am then
					playerkillplus()
				end
				if botleft < botn then
					botn = botleft
		    			bottab[bot.i] = nil
					if botleft == 0 then
						winstatehandler()
					end
				elseif bot.atype == "bot" then
					local spawn = math.random(1,#spawnmap)
					bot.grid_x = spawnmap[spawn].x*32
					bot.grid_y = spawnmap[spawn].y*32
        				bot.act_x = spawnmap[spawn].x*32
					bot.act_y = spawnmap[spawn].y*32
					bot.maxhp = math.random(1,math.max(player.level-1,1))
					bot.hp = bot.maxhp
				end
			end
end

function failconditionfunc(ahp)
	if ahp <= 0 and gamestate == 2 then
		winstate = "FAIL"
		if playsound then
			souride:stop()
			souboom:stop()
			love.audio.play(souboom)
		end
		player.slip = 0
		restorecontroll (player)
		screentimer = 5
		gamestate = 5
		love.filesystem.remove("tank.sav")
	elseif ahp <= 0 then
		startmapeditor("resume")
	end
end

function damagePlayerBase(am)
	playerbase.hp = playerbase.hp - (am or 1)
	failconditionfunc(playerbase.hp)
end

function damageplayer (am)
		player.hp = player.hp - (am or 1)
			if playsound then
				souhit:stop()
				love.audio.play(souhit)
			end
			failconditionfunc(player.hp)
end

function screentimerfunc(dt)
	if tonumber(screentimer) and tonumber(screentimer) > 0 then
		screentimer = screentimer - dt
		screendt = math.floor(screentimer) + 1
	else
		screendt = "Press any key to continue"
	end
end

updatefunc[4] = function (dt)
	screentimerfunc(dt)
end

updatefunc[5] = function (dt)
	screentimerfunc(dt)
end

function love.update(dt)
	if updatefunc[gamestate] then
		updatefunc[gamestate](dt)
	end
end

curmenu = allmenu.menutab

local drawfunc = {}

drawfunc[0] = function() -- debug
	love.graphics.setColor( 0, 0, 0)
	for i,error in pairs(confprocesstab) do
		love.graphics.print(error or "error", 128, 10+i*12)
	end
end

drawfunc[1] = function() --menu
	love.graphics.setColor( 250, 150, 1)
	love.graphics.rectangle("fill", 0, height - 128, width,128)
	love.graphics.rectangle("fill", width/2 -  4 * 32,48, 8*32,32)
	love.graphics.setColor( 0, 0, 0)
	love.graphics.print(navtab.desc or "error", 128, height - 120)
	love.graphics.print(curmenu[menustate].desc or "error", 128, height - 108)
	love.graphics.setColor( 200, 100, 1)
    	love.graphics.rectangle("fill",width/2 -  4 * 32, 16 + menustate * 32,8 * 32, 32)
    	love.graphics.setColor( 0, 0, 0)
	for i, node in pairs(curmenu) do
		love.graphics.print(node.dysplay or "error", width/2 -  3 * 32, 32 + i * 32 - 10)
	end

end
	
drawfunc[6] = function() -- dysplay score
	love.graphics.setColor( 250, 150, 1)
	love.graphics.rectangle("fill", width/2 -  4 * 32,48, 8*32,#scoretab*32+32)
	local pair = false
	love.graphics.setColor( 0, 0, 0)
	love.graphics.print("HIGH SCORES:", width/2 -  3 * 32, 2*32)
	love.graphics.setColor( 200, 100, 1)
	for i=1, #scoretab do
		if pair then
			love.graphics.rectangle("fill",width/2 -  4 * 32, 16 + i * 32,8 * 32, 32)
		end
		pair = (not pair)
	end
	love.graphics.setColor( 0, 0, 0)
	for i=1, #scoretab do
		local name = ""
		local ascore = ""
		for n,s in pairs(scoretab[i]) do
			name = n
			ascore = s
		end
		love.graphics.print(i..") "..name..": "..ascore, width/2 -  3 * 32, 32 + i * 32 +22)
	end
end

drawfunc[7] = function() -- dysplay score
	love.graphics.setColor( 250, 150, 1)
	love.graphics.rectangle("fill", width/2 -  4 * 32,48, 8*32,32)
	local pair = false
	love.graphics.setColor( 0, 0, 0)
	love.graphics.print("ENTER YOUR NAME:", width/2 -  3 * 32, 2*32)
	love.graphics.setColor( 200, 100, 1)
	love.graphics.rectangle("fill",width/2 -  4 * 32, 48+32,8 * 32, 32)
	love.graphics.setColor( 0, 0, 0)
	love.graphics.print(player.name..cursordysplay, width/2 -  3 * 32, 2 * 32 +22)
end

drawfunc[9] = function() -- editor name text
	love.graphics.setColor( 250, 150, 1)
	love.graphics.rectangle("fill", width/2 -  4 * 32,48, 8*32,32)
	local pair = false
	love.graphics.setColor( 0, 0, 0)
	love.graphics.print("ENTER MAP NAME:", width/2 -  3 * 32, 2*32)
	love.graphics.setColor( 200, 100, 1)
	love.graphics.rectangle("fill",width/2 -  4 * 32, 48+32,8 * 32, 32)
	love.graphics.setColor( 0, 0, 0)
	love.graphics.print((editorcursor.name or "")..cursordysplay, width/2 -  3 * 32, 2 * 32 +22)
	love.graphics.print((editorcursor.errorname or ""), width/2 -  3 * 32, 3 * 32 +22)
end
--local mapidtab = {[-2]=imagebot,[-1]=imageplayer,[1]=imagewat,[2]=imagewall,[3]=imagebri1,[4]=imagebri2,[5]=imagebri3,[6]=imageice,[7]=imagebus,[8]=imagemud,[9]=imagelaw,[10]=imagefir}
drawfunc[8] = function() --editor
	
	camera:set()
	for y=1, #map do
		
		for x=1, #map[y] do
			if  isElementOnScreen(x*32,y*32) then
				if mapidtab[map[y][x]] and mapidtab[map[y][x]] ~= 0 then
					if y == editorcursor.y and x == editorcursor.x then
						love.graphics.setColor(255,255,255,150)
						love.graphics.setColorMode("modulate")
					end
					love.graphics.drawq(imageatlas, mapidtab[map[y][x]], x*32+animcorect, y*32, 0, scalexbus, scaley, 0, 0 )
					--love.graphics.rectangle("line", x*32+animcorect, y*32, 32, 32)
					love.graphics.setColor(255,255,255,255)
					love.graphics.setColorMode("replace")
				end
				--love.graphics.line(32,x*32,#map*32,x*32)
		end
		--love.graphics.line(y*32,32,y*32,#map[y]*32)
        end
    end
	love.graphics.draw(imagegrid,0,0)
	if editorcursor.blinkstate then
		love.graphics.drawq(imageatlas, mapidtab[editorcursor.id], editorcursor.x*32+animcorect, editorcursor.y*32, 0, scalexbus, scaley, 0, 0 )
	end
	love.graphics.rectangle("line", editorcursor.x*32, editorcursor.y*32, 32, 32)
	--love.graphics.print(#editorcursor.botspawn.."      "..#editorcursor.plaspawn, editorcursor.x*32, editorcursor.y*32)
	camera:unset()
	love.graphics.print("FPS: "..love.timer.getFPS(), width -  4 * 32, height -12 * 20)
end

drawfunc[10] = function ()
	drawfunc[2]()
end

drawfunc[11] = function()
	love.graphics.draw(imagelogo1,0,0)
	love.graphics.draw(imagelogo2,512,0,0,0.5,0.5)
	love.graphics.draw(imagelogo3,256,0,0,0.5,0.5)
	love.graphics.setColor( 0, 0, 0)
	local texttab = {	"This game is made using love2d engine: https://love2d.org/",
				"Love uses scripting language lua: http://www.lua.org/",
				"This game is inspired by an old nes game called \"battle city\" (\"tanks\" in estern Europe)",
				"",
				"Authors:",
				"Lua scripting, level design and \"art*\" by Wojak",
				"Sound effects downloaded from: http://www.freesfx.co.uk/",
				"Special thanks to:",
				"Michael Ebens - camera module",
				"Tentus - Tutorial: Gridlocked Player",
				"",
				"Programs used:",
				"Scripting: Notepad++, MS Notepad",
				"Graphics: MS Paint, Pixelformer",
				"",
				"You can find more info in the readme file",
				"Contact e-mail: michal-020@o2.pl",
				"",
				"",
				"",
				"",
				"",
				"*except the logos on this screen"
				}
	for i,text in ipairs(texttab) do
		love.graphics.print(text,(width/2)-(7*#text/2),128+i*16)
	end
	love.graphics.setColor( 255, 255, 255)
end

drawfunc[2] = function() --game
	camera:set()
	love.graphics.draw(imagebackground,0,0)
	for y=1, #map do
		for x=1, #map[y] do
			if  isElementOnScreen(x*32,y*32) then
				if map[y][x] == 1 then
					love.graphics.drawq(imageatlas, imagewat, x*32+animcorect, y*32, 0, scalexbus, scaley, 0, 0 )
				--elseif map[y][x] == 2 then
					--love.graphics.drawq(imageatlas, imagewall, x*32, y*32, 0, scalex, scaley, 0, 0 )
				elseif map[y][x] == 3 then
					love.graphics.drawq(imageatlas, imagebri1, x*32, y*32, 0, scalex, scaley, 0, 0 )
				elseif map[y][x] == 4 then
					love.graphics.drawq(imageatlas, imagebri2, x*32, y*32, 0, scalex, scaley, 0, 0 )
				elseif map[y][x] == 5 then
					love.graphics.drawq(imageatlas, imagebri3, x*32, y*32, 0, scalex, scaley, 0, 0 )
				elseif map[y][x] == 6 then
					love.graphics.drawq(imageatlas, imageice, x*32, y*32, 0, scalex, scaley, 0, 0 )
				elseif map[y][x] == 8 then
					love.graphics.drawq(imageatlas, imagemud, x*32+animcorect, y*32, 0, scalexbus, scaley, 0, 0 )	
				elseif map[y][x] == 9 then
					love.graphics.drawq(imageatlas, imagelaw, x*32+animcorect, y*32, 0, scalexbus, scaley, 0, 0 )
				elseif map[y][x] == 11 then
					love.graphics.drawq(imageatlas, imagebri4, x*32, y*32, 0, scalex, scaley, 0, 0 )
				elseif map[y][x] == 12 then
					love.graphics.drawq(imageatlas, imagebri4, x*32, y*32, 0, scalex, scaley, 0, 0 )
				elseif map[y][x] == -3 then
					love.graphics.drawq(imageatlas, iamgebasepla, x*32, y*32, 0, scalex, scaley, 0, 0 )
				--elseif map[y][x] == 0 then
					--love.graphics.drawq(imageatlas, imagegras, x*32, y*32, 0, scalex, scaley, 0, 0 )
						
           		end
		 	end
        end
    end
	love.graphics.drawq(imageatlas, imageplayer,player.act_x + player.draw_x,player.act_y + player.draw_y, math.rad(player.direction*90), scalex, scaley, 0, 0 )
	love.graphics.setColor( 0, 0, 0)
	love.graphics.print("HP: "..player.hp.."/"..player.maxhp, player.act_x, player.act_y-15)
	love.graphics.print("dir: "..player.grid_dir.."  state: "..tostring(player.ismoving.overal).." "..tostring(player.slip).." "..tostring(player.slipstate), player.act_x, player.act_y-15-15)
	if (gamemodestate == 2 or gamemodestate == 4) then
		love.graphics.print(playerbase.hp,playerbase.x*32+10,playerbase.y*32+8)
	end
	love.graphics.setColor( 255, 255, 255)
	--love.graphics.print("         "..love.timer.getFPS()..", "..text2, player.act_x, player.act_y-20)
    	for i,bot in pairs( bottab )do
	  	if isElementOnScreen(bot.act_x,bot.act_y) then
			love.graphics.drawq(imageatlas, imagebot,bot.act_x + bot.draw_x,bot.act_y + bot.draw_y, math.rad(bot.direction*90), scalex, scaley, 0, 0 )
			love.graphics.setColor( 0, 0, 0)
			love.graphics.print("dir: "..bot.grid_dir.."  state: "..tostring(bot.ismoving.overal), bot.act_x, bot.act_y-15-15)
			love.graphics.print("HP: "..bot.hp.."/"..bot.maxhp, bot.act_x, bot.act_y-15)
			love.graphics.setColor( 255, 255, 255)
	  	end
    	end
	if (gamemodestate == 3 or gamemodestate == 4) then
		for i,base in pairs(basetab)  do
	  		if isElementOnScreen(base.act_x,base.act_y) then
				love.graphics.drawq(imageatlas, imagebasebot,base.act_x + base.draw_x,base.act_y + base.draw_y, math.rad(base.direction*90), scalex, scaley, 0, 0 )
				love.graphics.setColor( 0, 0, 0)
				love.graphics.print(base.hp,base.act_x+10,base.act_y+8)
				love.graphics.setColor( 255, 255, 255)
	  		end
		end
	end
	for i,bullet in pairs(bullettab)  do
	  	if isElementOnScreen(bullet.act_x,bullet.act_y) then
			love.graphics.circle( "fill",bullet.act_x + bullet.draw_x,bullet.act_y + bullet.draw_y, 6, 10 )
	  	end
	end
	for y=1, #map do
		for x=1, #map[y] do
			if  isElementOnScreen(x*32,y*32) then
				if map[y][x] == 7 then
					love.graphics.drawq(imageatlas, imagebus, x*32, y*32, 0, scalex, scaley, 0, 0 )
				elseif map[y][x] == 10 then
					love.graphics.drawq(imageatlas, imagefir, x*32+animcorect, y*32, 0, scalexbus, scaley, 0, 0 )
				end
			end
		end
	end
    	love.graphics.setColor( 255, 255, 255, 150)
	--[[for y=1, #dynamicmap do
        	for x=1, #dynamicmap[y] do
            		if dynamicmap[y][x] == 2 then
               			love.graphics.rectangle("fill", x * 32, y * 32, 32, 32)
            		end
        	end
   	 end]]
	camera:unset()
	love.graphics.setColor( 1, 100, 255,230)
   	love.graphics.rectangle("fill",width -  4 * 32, height -8 * 32,4 * 32, 8 * 32)
   	love.graphics.setColor( 30, 1, 1)
	love.graphics.print("FPS: "..love.timer.getFPS(), width -  4 * 32, height -12 * 20)
	love.graphics.print("SCORE: "..player.score, width -  4 * 32, height -8 * 20)
   	love.graphics.print("Bullet: "..tostring(not player.bullet), width -  4 * 32, height -7 * 20)
	love.graphics.print("kills: "..player.kills, width -  4 * 32, height -6 * 20)
  	love.graphics.print("kills to level: "..player.nextlevel, width -  4 * 32, height -5 * 20)
   	love.graphics.print("level: "..player.level, width -  4 * 32, height -4 * 20)
	love.graphics.print("HP: "..player.hp.."/"..player.maxhp, width -  4 * 32, height -3 * 20)
	love.graphics.print("bots on map: "..botn, width -  4 * 32, height -2 * 20)
   	love.graphics.print("bots left: "..botleft, width -  4 * 32, height -1 * 20)
end

function drawscreentimerfunc(victoryorfail)
	drawfunc[2]()
	love.graphics.setColor( 200, 100, 1)
    	love.graphics.rectangle("fill",width/2 -  4 * 32, height/2,8 * 32, 2*32)
	love.graphics.setColor( 0, 0, 0)
	love.graphics.print(victoryorfail, width/2 -  3 * 32, height/2 +10)
	love.graphics.print(screendt, width/2 -  3 * 32, height/2 +42)
end

drawfunc[4] = function() --victory
	drawscreentimerfunc("VICTORY!")
end

drawfunc[-1] = function() --restore
	drawfunc[1]()
end

drawfunc[5] = function() --fail
	drawscreentimerfunc("FAIL!")
end

drawfunc[3] = function() --pause
	drawfunc[2]()
	love.graphics.setColor( 250, 150, 1)
	love.graphics.rectangle("fill", width/2 -  5 * 32 -12, 0, 10*32 + 24,172)
	love.graphics.setColor( 0, 250, 0)
	love.graphics.rectangle("fill",width/2 -  5 * 32,0,10*32,160)
	drawfunc[1]()
end

function love.draw()
	--love.graphics.setColor( 0, 250, 0)
	--love.graphics.rectangle("fill",0,0,width,height)
	love.graphics.setColor( 255, 255, 255)
	if drawfunc[gamestate] then
		drawfunc[gamestate]()
	end
end

function setmapdrawtext(mapid)
	mapdraw = string.sub(maps[mapid],1,-5)
end
    

function love.keyreleased(key)
    if key == mp1.up and player and player.slip ==0 then
	isup = false
	udt = 0
	pstate = 0
	--souride:stop()
    elseif key == mp1.down and player and player.slip ==0 then
	isdown = false
	ddt = 0
	pstate = 0
	--souride:stop()
    elseif key == mp1.left and player and player.slip ==0 then
	isleft = false
	ldt = 0
	pstate = 0
	--souride:stop()
    elseif key == mp1.right and player and player.slip ==0 then
	isright = false
	rdt = 0
	pstate = 0
	--souride:stop()
    elseif key == mm0.action then
	nmenu = false
    end
    if key == "rshift" or key == "lshift" then
	shiftstate = false
    end
    
end

function didbullethit(bullet,bot)
	local bulpoints = {{x=-6,y=-6},{x=6,y=-6},{x=-6,y=6},{x=6,y=6}}
	--local returnval = false
	for i=1,4 do
		if (((bullet.act_y + bullet.draw_y + bulpoints[i].y) > (bot.act_y)) and ((bullet.act_y + bullet.draw_y + bulpoints[i].y) < (bot.act_y +32)) and ((bullet.act_x + bullet.draw_x + bulpoints[i].x) > (bot.act_x )) and ((bullet.act_x + bullet.draw_x + bulpoints[i].x) < (bot.act_x  +32))) then
			return true
			--break
		end
	end
	return false
end

function restorecontroll (bot)
	bot.slipstate = false
	isup = love.keyboard.isDown(mp1.up)
	isdown = love.keyboard.isDown(mp1.down)
	isleft = love.keyboard.isDown(mp1.left)
	isright = love.keyboard.isDown(mp1.right)
	if isup then bot.grid_dir = 0
	elseif isdown then bot.grid_dir = 2
	elseif isleft then bot.grid_dir = 3
	elseif isright then bot.grid_dir = 1 end
end

function testBotMap(bot,x, y) --(edit-30-03-12)
    local id = 0
    local dynamicid = 0
	local id2 = 0
    if map[(bot.grid_y / 32) + y] and map[(bot.grid_y / 32) + y][(bot.grid_x / 32) + x] then 
		id = map[(bot.grid_y / 32) + y][(bot.grid_x / 32) + x]
		dynamicid = dynamicmap[(bot.grid_y / 32) + y][(bot.grid_x / 32) + x]
		if map[(bot.grid_y / 32) + 2*y] and map[(bot.grid_y / 32) + 2*y][(bot.grid_x / 32) + 2*x] then 
			id2 = map[(bot.grid_y / 32) + 2*y][(bot.grid_x / 32) + 2*x]
		end
    else
		return false
    end 
    if hasidslowdowneffect[id] and bot.atype ~= "base" then
		bot.speed = hasidslowdowneffect[id]
    else
		bot.speed = bot.basespeed
    end
    if	hasidsliperyeffect[id] and bot.slip ==0 then
	bot.slip = hasidsliperyeffect[id]
        if bot.atype == "player" then bot.slipstate = true end
    elseif hasidsliperyeffect[id] and bot.slip ~= 0 then
		bot.slip = bot.slip - 1
		if bot.atype == "player" then 
			bot.slipstate = true 
			if bot.slip == 0 then
				restorecontroll (bot)
			end
		end
    else
		bot.slip = 0
		if bot.atype == "player" and bot.slipstate then 
			restorecontroll (bot)
		end
    end
	if hasiddamegeeffect[id] then
		if bot.atype == "player" and firedetectordisplay == "OFF" then
			damageplayer(hasiddamegeeffect[id])
		elseif bot.atype == "base" then
			return false
		elseif (bot.hp > hasiddamegeeffect[id]) and (not hasiddamegeeffect[id2]) then
			local rand = math.random(1,2)
			if rand == 1 then 
				if bot.atype == "player" then
					damageplayer(hasiddamegeeffect[id])
				else	
					damagebot(bot,hasiddamegeeffect[id]) 
				end
			elseif rand == 2 and bot.atype == "player" then
				damageplayer(hasiddamegeeffect[id])
			else
				if bot.atype == "player" then 
					bot.slip = 0
					bot.slipstate = false
					isup = false
					isdown = false
					isleft = false
					isright = false
				end
				return false
			end
		else
			if bot.atype == "player" then 
				bot.slip = 0
				bot.slipstate = false
				isup = false
				isdown = false
				isleft = false
				isright = false
			end
			return false
		end
	end
    if hasidcolision[id] or dynamicid == 1 then
		if isidbrick[id] and bot.atype == "bot" and not bot.bullet then
			local abullet = createBullet(bot.i,bot.grid_x,bot.grid_y,bot.direction)
			bot.bullet = true
			bullettab[bot.i] = abullet
			rotOfsetBullet(abullet)
		end
		return false
    elseif (bot.atype == "bot" or bot.atype == "base") and (dynamicid == 2 or dynamicid == 3)then
		return false
    end
		return true
end
function testbulMap(bot,x, y,tx,ty,lx,ly,rx,ry) --(edit-30-03-12)
    local id = 0
    local dynamicid = {}
    if map[(bot.grid_y / 32)+y] and (map[(bot.grid_y / 32)+y][(bot.grid_x / 32)]+x) then
   	id = (map[(bot.grid_y / 32)+y][(bot.grid_x / 32)+x])
	dynamicid[1] = dynamicmap[(bot.grid_y / 32)+y][(bot.grid_x / 32)+x]
	dynamicid[5] = dynamicmap[(bot.grid_y / 32)][(bot.grid_x / 32)]
	if map[(bot.grid_y / 32)+ty] and (map[(bot.grid_y / 32)+ty][(bot.grid_x / 32)]+tx) then
		dynamicid[2] = dynamicmap[(bot.grid_y / 32)+ty][(bot.grid_x / 32)+tx]
	end
	if map[(bot.grid_y / 32)+ly] and (map[(bot.grid_y / 32)+ly][(bot.grid_x / 32)]+lx) then
		dynamicid[3] = dynamicmap[(bot.grid_y / 32)+ly][(bot.grid_x / 32)+lx]
	end
	if map[(bot.grid_y / 32)+ry] and (map[(bot.grid_y / 32)+ry][(bot.grid_x / 32)]+rx) then
	
		dynamicid[4] = dynamicmap[(bot.grid_y / 32)+ry][(bot.grid_x / 32)+rx]
	end
    else
	return false
    end
    bot.acurat = false
    for i=1,#dynamicid do
	if bot.botid ~= "pla" and isdynamicidplayer[dynamicid[i]] then 
		bot.acurat = true 
		break
	elseif bot.botid == "pla" and isdynamicidbot[dynamicid[i]] then 
		bot.acurat = true 
		break
	end
    end
    if hasidcolisionforbullets[id] then
		if id == 11 then
			map[(bot.grid_y / 32)+y][(bot.grid_x / 32)+x] = 12
		elseif id == 12 then
			map[(bot.grid_y / 32)+y][(bot.grid_x / 32)+x] = 3
		elseif id == 3 then
			map[(bot.grid_y / 32)+y][(bot.grid_x / 32)+x] = 4
		elseif id == 4 then
			map[(bot.grid_y / 32)+y][(bot.grid_x / 32)+x] = 5
		elseif id == 5 then
			map[(bot.grid_y / 32)+y][(bot.grid_x / 32)+x] = 0
		elseif id == -3 and bot.botid ~= "pla" then
			damagePlayerBase(1)
		end 
        return false
    end
    return true
end
function isElementOnScreen(x,y)
	return (x > camera._x - 64) and (x < camera._x + width + 64) and (y > camera._y - 64) and (y < camera._y + height + 64)
end

local keyfunc = {}
function love.keypressed(key)
	if keyfunc[gamestate] then keyfunc[gamestate](key) end
end
keyfunc[1] = function(key) -- menu
	if readkey then
		readkey[1][readkey[2]] = key
		reloadmenu()
		if readkey[1] == mp1 then
			curmenu = allmenu.player1keysettingtab
		elseif readkey[1] == me0 then
			curmenu = allmenu.editorkeysettingtab
		elseif readkey[1] == mm0 then
			curmenu = allmenu.menukeysettingtab
		end
		readkey = false
	elseif navtab[key] then navtab[key]()
	elseif curmenu[menustate][key] then curmenu[menustate][key]() end
end

keyfunc[10] = function (key)
	keyfunc[2](key)
end

keyfunc[2] = function (key) -- game
	if movetab[key] then movetab[key]() end
end
keyfunc[0] = function (key) -- error
	gamestate = -1
	curmenu = allmenu.restoretab
end
function postgame(func)
	if not (screentimer > 0) then
		func()
	end
end

function nextmap()
	botleft = storebotleft + botincreese
	botincreese = botincreese + botincreeseamount
	--botn = dificulitytab[2].botn
	mapid = mapid + 1
	if mapid > #maps then mapid =1 end
	setmapdrawtext(mapid)
	--suboptions[3].dysplay = "MAP: "..mapdraw -- warnirg
	startgame("next",botleft)
end

function savescore_a()
	local user,ascore
	for u,s in pairs(scoretab[10]) do
		user = u
		ascore = s
	end
	if player.score > ascore then
		gamestate = 7
	else
		startmenu()
	end
end

function savescore()
	scoretab[11] = {[player.name] = player.score}
	local state = true
	local loop = 0
	while state do
  		loop = loop + 1
  		
  		state = false
  		for i,n in pairs(scoretab) do
   			local name,val,name1,val1
    			for j,v in pairs(n) do
      				name = j
      				val = v
    			end
    			if scoretab[i+1] then
    				for j,v in pairs(scoretab[i+1]) do
        				name1 = j
       			 		val1 = v
      				end
      				if val < val1 then 
        				scoretab[i][name]=nil
        				scoretab[i][name1]=val1
        				scoretab[i+1][name1]=nil
        				scoretab[i+1][name]=val
        				--print(name.." = "..val.." < "..name1.." = "..val1)
       					state = true
      
     				 end
    			end
  		end
	end
	scoretab[11] = nil
	local text = "return "..table.tostring(scoretab) 
	love.filesystem.write("score.txt",text)
	gamestate = 6
end

specialkeytab = {
	backspace = function()
		local tab = {}
		if gamestate == 7 then 
			tab = player
		elseif gamestate == 9 then 
			tab = editorcursor
		end
		local l = string.len(tab.name or "")
		if l > 0 then
			tab.name = string.sub(tab.name,0,l-1)
		end
	end,
	lshift = function()
		shiftstate = true
	end,
	rshift = function()
		shiftstate = true
	end,
	["return"] = function()
		if gamestate == 7 then
			local l = string.len(player.name)
			if l > 0 then
				savescore()
			end
		elseif gamestate == 9 then
			local l = string.len(editorcursor.name or "")
			if l == 0 then
				editorcursor.errorname = "To short name!"
			elseif tonumber(editorcursor.name) then
				editorcursor.errorname = "Name can't be a number!"
			else 
				local maplistbackupfunc = loadfile(savedir.."/maplist.bck") 
				local maplistbackup = maplistbackupfunc()
				local overstate = true
				for i,amap in pairs (maplistbackup) do
					amap = string.sub(amap,1,-5)
					if amap == editorcursor.name then
						editorcursor.errorname = ""
						love.graphics.setColorMode("modulate")
						startmenu()
						curmenu = allmenu.doyouwantoverwritetab
						menustate = 2
						overstate = false
						break
					end
				end
				if overstate then
					local savetext = "return "..table.tostring(map)
					savetext = string.gsub (savetext,"{{","{\n{") 
					savetext = string.gsub (savetext,"},",",},\n")
					savetext = string.gsub (savetext,"}}","}\n}")
					local newfile = io.open(directory.."/"..editorcursor.name..".map", "w")
					newfile:write(savetext)
					newfile:close()

					table.insert(maplistbackup,#maplistbackup + 1,editorcursor.name..".map")						

					actualconfig.maps = maplistbackup
					local savetext = "return "..table.tostring(actualconfig)
					savetext = string.gsub (savetext,"{","{\n") 
					savetext = string.gsub (savetext,",",",\n")
					savetext = string.gsub (savetext,"}","\n}")
					savetext = string.gsub (savetext,"return {","return {-- acceptable keys: \"0\", \"1\", \"2\", \"3\", \"4\", \5\", \"6\", \"7\", \"8\", \"9\", \"a\", \"b\", \"c\", \"d\", \"e\", \"f\", \"g\", \"h\", \"i\", \"j\", \"k\",\"l\", \"m\", \"n\", \"o\", \"p\", \"q\", \"r\", \"s\", \"t\", \"u\", \"v\", \"w\", \"x\", \"y\", \"z\",\"F1\", \"F2\", \"F3\", \"F4\", \"F5\",\"F6\", \"F7\", \"F8\", \"F9\", \"F10\", \"F11\", \"F12\", \"backspace\", \"tab\", \"lalt\", \"ralt\", \"return\", \" \", \"pageup\", \"pagedown\", \"end\", \"home\",\"insert\", \"delete\", \"lshift\", \"rshift\", \"lctrl\", \"rctrl\", \"[\", \"]\", \"pause\", \"escape\", \"capslock\", \"scroll\", \";\", \",\", \"-\", \".\", \"/\",\"#\", \"\\\", \"=\",\"up\", \"down\", \"left\", \"right\", \"kp0\",\"kp1\",\"kp2\",\"kp3\",\"kp4\",\"kp5\",\"kp6\",\"kp7\",\"kp8\",\"kp9\",\"kp.\",\"kp/\",\"kp*\",\"kp-\",\"kp+\",\"kpenter\",\"kp=\"")
					local newfile = io.open(directory.."/tank.conf", "w")
					newfile:write(savetext)
					newfile:close()
					loadconfigfile()
				end
			end
		end
	end
}

function typenamefunc(tab,key)
	local l = string.len(tab.name or "")
	if l < 16 then
		if shiftstate then key = string.upper(key) end
		tab.name = (tab.name or "")..key
	end

end

keyfunc[4] = function (key) -- victory
	postgame(nextmap)
end

keyfunc[5] = function (key) -- fail
	postgame(savescore_a)
end

keyfunc[6] = function (key) -- fail
	startmenu()
end

keyfunc[11] = function (key) -- fail
	imagelogo1 = nil
	imagelogo2 = nil
	imagelogo3 = nil
	startmenu()
end

keyfunc[7] = function (key) -- fail
	if specialkeytab[key] then specialkeytab[key]()
	elseif iskeytab[key] then typenamefunc(player,key) end
end

keyfunc[9] = function (key)
	if specialkeytab[key] then specialkeytab[key]()
	elseif iskeytab[key] then typenamefunc(editorcursor,key) end
end

keyfunc[3] = function (key) -- pause love.mousepressed( x, y, button )
	keyfunc[1](key)
end

keyfunc[8] = function (key) -- editor
	editorkeys(key)
end

keyfunc[-1] = function (key) -- restore
	keyfunc[1](key)
end

function love.focus(f)
	if not f and gamestate == 2 then
		if playsound then
			souride:stop()
		end
		gamestate = 3
		curmenu = pausemenutab 
		menustate = 2
	end
end
loadconfigfile()